@@ -1,16 +1,17 @@
 ;; Foreign functions provided by the Linux C library version 6,
 ;; i.e. the GNU C library version 2.0.7.
 ;; Bruno Haible 10.4.1998, 19.4.1998
-;; Sam Steingold 2002-2005
+;; Sam Steingold 2002-2008
 
 (defpackage "LINUX"
   (:modern t)
   (:nicknames "UNIX" "GLIBC")
   (:use "COMMON-LISP" "FFI")
   (:shadowing-import-from "EXPORTING"
-           #:defconstant #:defun #:defmacro #:define-modify-macro
-           #:define-symbol-macro #:defsetf
-           #:def-c-type #:def-c-enum #:def-c-struct #:def-c-var #:def-call-out)
+    #:defconstant #:defun #:defmacro #:define-modify-macro
+    #:define-symbol-macro #:defsetf
+    #:def-c-type #:def-c-enum #:def-c-struct #:def-c-var
+    #:def-c-const #:def-call-out)
   (:shadow read write random abort abs acos asin atan cos sin tan cosh sinh tanh
            acosh asinh atanh exp log sqrt floor truncate ftruncate open close
            remove sleep))
@@ -30,11 +31,9 @@
 
 (c-lines "#include <stddef.h>~%")
 
-(def-c-type ptrdiff_t long)
+(def-c-type ptrdiff_t)          ; long
 
-(def-c-type size_t uint)  ; uint or ulong, doesn't matter
-
-(def-c-type wchar_t int)
+(def-c-type wchar_t)            ; int
 
 ; =========================== <sys/types.h> ===================================
 
@@ -50,25 +49,25 @@
 (def-c-type __quad_t sint64)
 (def-c-type __qaddr_t (c-ptr __quad_t))
 
-(def-c-type __dev_t __u_quad_t)
-(def-c-type __uid_t __u_int)
-(def-c-type __gid_t __u_int)
-(def-c-type __ino_t __u_long)
-(def-c-type __mode_t __u_int)
-(def-c-type __nlink_t __u_int)
-(def-c-type __off_t long)
-(def-c-type __loff_t __quad_t)
-(def-c-type __pid_t int)
-(def-c-type __ssize_t int)
+(def-c-type __dev_t)            ; __u_quad_t
+(def-c-type __uid_t)            ; __u_int
+(def-c-type __gid_t)            ; __u_int
+(def-c-type __ino_t)            ; __u_long
+(def-c-type __mode_t)           ; __u_int
+(def-c-type __nlink_t)          ; __u_int
+(def-c-type __off_t)            ; long
+(def-c-type __loff_t)           ; __quad_t
+(def-c-type __pid_t)            ; int
+(def-c-type __ssize_t)          ; int
 
 (def-c-struct __fsid_t
   (__val (c-array int 2))
 )
 
-(def-c-type __daddr_t int)
-(def-c-type __caddr_t c-pointer)
-(def-c-type __time_t long)
-(def-c-type __swblk_t long)
+(def-c-type __daddr_t)          ; int
+(def-c-type __caddr_t)          ; c-pointer
+(def-c-type __time_t)           ; long
+(def-c-type __swblk_t)          ; long
 
 (def-c-type __fd_mask ulong)
 (eval-when (load compile eval)
@@ -85,24 +84,25 @@
   (fds_bits (c-array __fd_mask #.(cl:/ __FD_SETSIZE __NFDBITS)))
 )
 
-(def-c-type __key_t int)
+(def-c-type __key_t)            ; int
 
-(def-c-type __ipc_pid_t ushort)
+(c-lines "#include <bits/ipctypes.h>~%")
+(def-c-type __ipc_pid_t)        ; ushort
 
 ; --------------------------- <sys/types.h> -----------------------------------
 
 (def-c-type dev_t __dev_t)
 (def-c-type gid_t __gid_t)
 (def-c-type ino_t __ino_t)
+(def-c-type ino64_t)
 (def-c-type mode_t __mode_t)
 (def-c-type nlink_t __nlink_t)
 (def-c-type off_t __off_t)
+(def-c-type off64_t)
 (def-c-type loff_t __loff_t)
 (def-c-type pid_t __pid_t)
 (def-c-type uid_t __uid_t)
 
-(def-c-type ssize_t __ssize_t)
-
 (def-c-type daddr_t __daddr_t)
 (def-c-type caddr_t __caddr_t)
 
@@ -110,8 +110,6 @@
 
 (def-c-type time_t __time_t)   ; in <time.h>
 
-(def-c-type size_t uint)       ; in <stddef.h>
-
 (def-c-type int8_t sint8)
 (def-c-type u_int8_t uint8)
 (def-c-type int16_t sint16)
@@ -163,130 +161,130 @@
 
 ; ---------------------------- <asm/errno.h> ----------------------------------
 
-(defconstant    EPERM            1)     ; Operation not permitted
-(defconstant    ENOENT           2)     ; No such file or directory
-(defconstant    ESRCH            3)     ; No such process
-(defconstant    EINTR            4)     ; Interrupted system call
-(defconstant    EIO              5)     ; I/O error
-(defconstant    ENXIO            6)     ; No such device or address
-(defconstant    E2BIG            7)     ; Arg list too long
-(defconstant    ENOEXEC          8)     ; Exec format error
-(defconstant    EBADF            9)     ; Bad file number
-(defconstant    ECHILD          10)     ; No child processes
-(defconstant    EAGAIN          11)     ; Try again
-(defconstant    ENOMEM          12)     ; Out of memory
-(defconstant    EACCES          13)     ; Permission denied
-(defconstant    EFAULT          14)     ; Bad address
-(defconstant    ENOTBLK         15)     ; Block device required
-(defconstant    EBUSY           16)     ; Device or resource busy
-(defconstant    EEXIST          17)     ; File exists
-(defconstant    EXDEV           18)     ; Cross-device link
-(defconstant    ENODEV          19)     ; No such device
-(defconstant    ENOTDIR         20)     ; Not a directory
-(defconstant    EISDIR          21)     ; Is a directory
-(defconstant    EINVAL          22)     ; Invalid argument
-(defconstant    ENFILE          23)     ; File table overflow
-(defconstant    EMFILE          24)     ; Too many open files
-(defconstant    ENOTTY          25)     ; Not a typewriter
-(defconstant    ETXTBSY         26)     ; Text file busy
-(defconstant    EFBIG           27)     ; File too large
-(defconstant    ENOSPC          28)     ; No space left on device
-(defconstant    ESPIPE          29)     ; Illegal seek
-(defconstant    EROFS           30)     ; Read-only file system
-(defconstant    EMLINK          31)     ; Too many links
-(defconstant    EPIPE           32)     ; Broken pipe
-(defconstant    EDOM            33)     ; Math argument out of domain of func
-(defconstant    ERANGE          34)     ; Math result not representable
-(defconstant    EDEADLK         35)     ; Resource deadlock would occur
-(defconstant    ENAMETOOLONG    36)     ; File name too long
-(defconstant    ENOLCK          37)     ; No record locks available
-(defconstant    ENOSYS          38)     ; Function not implemented
-(defconstant    ENOTEMPTY       39)     ; Directory not empty
-(defconstant    ELOOP           40)     ; Too many symbolic links encountered
-(defconstant    EWOULDBLOCK     EAGAIN) ; Operation would block
-(defconstant    ENOMSG          42)     ; No message of desired type
-(defconstant    EIDRM           43)     ; Identifier removed
-(defconstant    ECHRNG          44)     ; Channel number out of range
-(defconstant    EL2NSYNC        45)     ; Level 2 not synchronized
-(defconstant    EL3HLT          46)     ; Level 3 halted
-(defconstant    EL3RST          47)     ; Level 3 reset
-(defconstant    ELNRNG          48)     ; Link number out of range
-(defconstant    EUNATCH         49)     ; Protocol driver not attached
-(defconstant    ENOCSI          50)     ; No CSI structure available
-(defconstant    EL2HLT          51)     ; Level 2 halted
-(defconstant    EBADE           52)     ; Invalid exchange
-(defconstant    EBADR           53)     ; Invalid request descriptor
-(defconstant    EXFULL          54)     ; Exchange full
-(defconstant    ENOANO          55)     ; No anode
-(defconstant    EBADRQC         56)     ; Invalid request code
-(defconstant    EBADSLT         57)     ; Invalid slot
-(defconstant    EDEADLOCK       EDEADLK); File locking deadlock error
-(defconstant    EBFONT          59)     ; Bad font file format
-(defconstant    ENOSTR          60)     ; Device not a stream
-(defconstant    ENODATA         61)     ; No data available
-(defconstant    ETIME           62)     ; Timer expired
-(defconstant    ENOSR           63)     ; Out of streams resources
-(defconstant    ENONET          64)     ; Machine is not on the network
-(defconstant    ENOPKG          65)     ; Package not installed
-(defconstant    EREMOTE         66)     ; Object is remote
-(defconstant    ENOLINK         67)     ; Link has been severed
-(defconstant    EADV            68)     ; Advertise error
-(defconstant    ESRMNT          69)     ; Srmount error
-(defconstant    ECOMM           70)     ; Communication error on send
-(defconstant    EPROTO          71)     ; Protocol error
-(defconstant    EMULTIHOP       72)     ; Multihop attempted
-(defconstant    EDOTDOT         73)     ; RFS specific error
-(defconstant    EBADMSG         74)     ; Not a data message
-(defconstant    EOVERFLOW       75)     ; Value too large for defined data type
-(defconstant    ENOTUNIQ        76)     ; Name not unique on network
-(defconstant    EBADFD          77)     ; File descriptor in bad state
-(defconstant    EREMCHG         78)     ; Remote address changed
-(defconstant    ELIBACC         79)     ; Can not access a needed shared library
-(defconstant    ELIBBAD         80)     ; Accessing a corrupted shared library
-(defconstant    ELIBSCN         81)     ; .lib section in a.out corrupted
-(defconstant    ELIBMAX         82)     ; Attempting to link in too many shared libraries
-(defconstant    ELIBEXEC        83)     ; Cannot exec a shared library directly
-(defconstant    EILSEQ          84)     ; Illegal byte sequence
-(defconstant    ERESTART        85)     ; Interrupted system call should be restarted
-(defconstant    ESTRPIPE        86)     ; Streams pipe error
-(defconstant    EUSERS          87)     ; Too many users
-(defconstant    ENOTSOCK        88)     ; Socket operation on non-socket
-(defconstant    EDESTADDRREQ    89)     ; Destination address required
-(defconstant    EMSGSIZE        90)     ; Message too long
-(defconstant    EPROTOTYPE      91)     ; Protocol wrong type for socket
-(defconstant    ENOPROTOOPT     92)     ; Protocol not available
-(defconstant    EPROTONOSUPPORT 93)     ; Protocol not supported
-(defconstant    ESOCKTNOSUPPORT 94)     ; Socket type not supported
-(defconstant    EOPNOTSUPP      95)     ; Operation not supported on transport endpoint
-(defconstant    EPFNOSUPPORT    96)     ; Protocol family not supported
-(defconstant    EAFNOSUPPORT    97)     ; Address family not supported by protocol
-(defconstant    EADDRINUSE      98)     ; Address already in use
-(defconstant    EADDRNOTAVAIL   99)     ; Cannot assign requested address
-(defconstant    ENETDOWN        100)    ; Network is down
-(defconstant    ENETUNREACH     101)    ; Network is unreachable
-(defconstant    ENETRESET       102)    ; Network dropped connection because of reset
-(defconstant    ECONNABORTED    103)    ; Software caused connection abort
-(defconstant    ECONNRESET      104)    ; Connection reset by peer
-(defconstant    ENOBUFS         105)    ; No buffer space available
-(defconstant    EISCONN         106)    ; Transport endpoint is already connected
-(defconstant    ENOTCONN        107)    ; Transport endpoint is not connected
-(defconstant    ESHUTDOWN       108)    ; Cannot send after transport endpoint shutdown
-(defconstant    ETOOMANYREFS    109)    ; Too many references: cannot splice
-(defconstant    ETIMEDOUT       110)    ; Connection timed out
-(defconstant    ECONNREFUSED    111)    ; Connection refused
-(defconstant    EHOSTDOWN       112)    ; Host is down
-(defconstant    EHOSTUNREACH    113)    ; No route to host
-(defconstant    EALREADY        114)    ; Operation already in progress
-(defconstant    EINPROGRESS     115)    ; Operation now in progress
-(defconstant    ESTALE          116)    ; Stale NFS file handle
-(defconstant    EUCLEAN         117)    ; Structure needs cleaning
-(defconstant    ENOTNAM         118)    ; Not a XENIX named type file
-(defconstant    ENAVAIL         119)    ; No XENIX semaphores available
-(defconstant    EISNAM          120)    ; Is a named type file
-(defconstant    EREMOTEIO       121)    ; Remote I/O error
-(defconstant    EDQUOT          122)    ; Quota exceeded
-(defconstant    ENOMEDIUM       123)    ; No medium found
-(defconstant    EMEDIUMTYPE     124)    ; Wrong medium type
+(def-c-const EPERM (:documentation "Operation not permitted"))
+(def-c-const ENOENT (:documentation "No such file or directory"))
+(def-c-const ESRCH (:documentation "No such process"))
+(def-c-const EINTR (:documentation "Interrupted system call"))
+(def-c-const EIO (:documentation "I/O error"))
+(def-c-const ENXIO (:documentation "No such device or address"))
+(def-c-const E2BIG (:documentation "Arg list too long"))
+(def-c-const ENOEXEC (:documentation "Exec format error"))
+(def-c-const EBADF (:documentation "Bad file number"))
+(def-c-const ECHILD (:documentation "No child processes"))
+(def-c-const EAGAIN (:documentation "Try again"))
+(def-c-const ENOMEM (:documentation "Out of memory"))
+(def-c-const EACCES (:documentation "Permission denied"))
+(def-c-const EFAULT (:documentation "Bad address"))
+(def-c-const ENOTBLK (:documentation "Block device required"))
+(def-c-const EBUSY (:documentation "Device or resource busy"))
+(def-c-const EEXIST (:documentation "File exists"))
+(def-c-const EXDEV (:documentation "Cross-device link"))
+(def-c-const ENODEV (:documentation "No such device"))
+(def-c-const ENOTDIR (:documentation "Not a directory"))
+(def-c-const EISDIR (:documentation "Is a directory"))
+(def-c-const EINVAL (:documentation "Invalid argument"))
+(def-c-const ENFILE (:documentation "File table overflow"))
+(def-c-const EMFILE (:documentation "Too many open files"))
+(def-c-const ENOTTY (:documentation "Not a typewriter"))
+(def-c-const ETXTBSY (:documentation "Text file busy"))
+(def-c-const EFBIG (:documentation "File too large"))
+(def-c-const ENOSPC (:documentation "No space left on device"))
+(def-c-const ESPIPE (:documentation "Illegal seek"))
+(def-c-const EROFS (:documentation "Read-only file system"))
+(def-c-const EMLINK (:documentation "Too many links"))
+(def-c-const EPIPE (:documentation "Broken pipe"))
+(def-c-const EDOM (:documentation "Math argument out of domain of func"))
+(def-c-const ERANGE (:documentation "Math result not representable"))
+(def-c-const EDEADLK (:documentation "Resource deadlock would occur"))
+(def-c-const ENAMETOOLONG (:documentation "File name too long"))
+(def-c-const ENOLCK (:documentation "No record locks available"))
+(def-c-const ENOSYS (:documentation "Function not implemented"))
+(def-c-const ENOTEMPTY (:documentation "Directory not empty"))
+(def-c-const ELOOP (:documentation "Too many symbolic links encountered"))
+(def-c-const EWOULDBLOCK (:documentation "Operation would block"))
+(def-c-const ENOMSG (:documentation "No message of desired type"))
+(def-c-const EIDRM (:documentation "Identifier removed"))
+(def-c-const ECHRNG (:documentation "Channel number out of range"))
+(def-c-const EL2NSYNC (:documentation "Level 2 not synchronized"))
+(def-c-const EL3HLT (:documentation "Level 3 halted"))
+(def-c-const EL3RST (:documentation "Level 3 reset"))
+(def-c-const ELNRNG (:documentation "Link number out of range"))
+(def-c-const EUNATCH (:documentation "Protocol driver not attached"))
+(def-c-const ENOCSI (:documentation "No CSI structure available"))
+(def-c-const EL2HLT (:documentation "Level 2 halted"))
+(def-c-const EBADE (:documentation "Invalid exchange"))
+(def-c-const EBADR (:documentation "Invalid request descriptor"))
+(def-c-const EXFULL (:documentation "Exchange full"))
+(def-c-const ENOANO (:documentation "No anode"))
+(def-c-const EBADRQC (:documentation "Invalid request code"))
+(def-c-const EBADSLT (:documentation "Invalid slot"))
+(def-c-const EDEADLOCK (:documentation "File locking deadlock error"))
+(def-c-const EBFONT (:documentation "Bad font file format"))
+(def-c-const ENOSTR (:documentation "Device not a stream"))
+(def-c-const ENODATA (:documentation "No data available"))
+(def-c-const ETIME (:documentation "Timer expired"))
+(def-c-const ENOSR (:documentation "Out of streams resources"))
+(def-c-const ENONET (:documentation "Machine is not on the network"))
+(def-c-const ENOPKG (:documentation "Package not installed"))
+(def-c-const EREMOTE (:documentation "Object is remote"))
+(def-c-const ENOLINK (:documentation "Link has been severed"))
+(def-c-const EADV (:documentation "Advertise error"))
+(def-c-const ESRMNT (:documentation "Srmount error"))
+(def-c-const ECOMM (:documentation "Communication error on send"))
+(def-c-const EPROTO (:documentation "Protocol error"))
+(def-c-const EMULTIHOP (:documentation "Multihop attempted"))
+(def-c-const EDOTDOT (:documentation "RFS specific error"))
+(def-c-const EBADMSG (:documentation "Not a data message"))
+(def-c-const EOVERFLOW (:documentation "Value too large for defined data type"))
+(def-c-const ENOTUNIQ (:documentation "Name not unique on network"))
+(def-c-const EBADFD (:documentation "File descriptor in bad state"))
+(def-c-const EREMCHG (:documentation "Remote address changed"))
+(def-c-const ELIBACC (:documentation "Can not access a needed shared library"))
+(def-c-const ELIBBAD (:documentation "Accessing a corrupted shared library"))
+(def-c-const ELIBSCN (:documentation ".lib section in a.out corrupted"))
+(def-c-const ELIBMAX (:documentation "Attempting to link in too many shared libraries"))
+(def-c-const ELIBEXEC (:documentation "Cannot exec a shared library directly"))
+(def-c-const EILSEQ (:documentation "Illegal byte sequence"))
+(def-c-const ERESTART (:documentation "Interrupted system call should be restarted"))
+(def-c-const ESTRPIPE (:documentation "Streams pipe error"))
+(def-c-const EUSERS (:documentation "Too many users"))
+(def-c-const ENOTSOCK (:documentation "Socket operation on non-socket"))
+(def-c-const EDESTADDRREQ (:documentation "Destination address required"))
+(def-c-const EMSGSIZE (:documentation "Message too long"))
+(def-c-const EPROTOTYPE (:documentation "Protocol wrong type for socket"))
+(def-c-const ENOPROTOOPT (:documentation "Protocol not available"))
+(def-c-const EPROTONOSUPPORT (:documentation "Protocol not supported"))
+(def-c-const ESOCKTNOSUPPORT (:documentation "Socket type not supported"))
+(def-c-const EOPNOTSUPP (:documentation "Operation not supported on transport endpoint"))
+(def-c-const EPFNOSUPPORT (:documentation "Protocol family not supported"))
+(def-c-const EAFNOSUPPORT (:documentation "Address family not supported by protocol"))
+(def-c-const EADDRINUSE (:documentation "Address already in use"))
+(def-c-const EADDRNOTAVAIL (:documentation "Cannot assign requested address"))
+(def-c-const ENETDOWN (:documentation "Network is down"))
+(def-c-const ENETUNREACH (:documentation "Network is unreachable"))
+(def-c-const ENETRESET (:documentation "Network dropped connection because of reset"))
+(def-c-const ECONNABORTED (:documentation "Software caused connection abort"))
+(def-c-const ECONNRESET (:documentation "Connection reset by peer"))
+(def-c-const ENOBUFS (:documentation "No buffer space available"))
+(def-c-const EISCONN (:documentation "Transport endpoint is already connected"))
+(def-c-const ENOTCONN (:documentation "Transport endpoint is not connected"))
+(def-c-const ESHUTDOWN (:documentation "Cannot send after transport endpoint shutdown"))
+(def-c-const ETOOMANYREFS (:documentation "Too many references: cannot splice"))
+(def-c-const ETIMEDOUT (:documentation "Connection timed out"))
+(def-c-const ECONNREFUSED (:documentation "Connection refused"))
+(def-c-const EHOSTDOWN (:documentation "Host is down"))
+(def-c-const EHOSTUNREACH (:documentation "No route to host"))
+(def-c-const EALREADY (:documentation "Operation already in progress"))
+(def-c-const EINPROGRESS (:documentation "Operation now in progress"))
+(def-c-const ESTALE (:documentation "Stale NFS file handle"))
+(def-c-const EUCLEAN (:documentation "Structure needs cleaning"))
+(def-c-const ENOTNAM (:documentation "Not a XENIX named type file"))
+(def-c-const ENAVAIL (:documentation "No XENIX semaphores available"))
+(def-c-const EISNAM (:documentation "Is a named type file"))
+(def-c-const EREMOTEIO (:documentation "Remote I/O error"))
+(def-c-const EDQUOT (:documentation "Quota exceeded"))
+(def-c-const ENOMEDIUM (:documentation "No medium found"))
+(def-c-const EMEDIUMTYPE (:documentation "Wrong medium type"))
 
 ; -------------------------- <bits/errno.h> -----------------------------------
 
@@ -1689,72 +1687,72 @@
 (defconstant SEEK_CUR 1)
 (defconstant SEEK_END 2)
 
-(def-c-type FILE (c-struct vector #| components unknown |# ))
+(def-c-type FILE c-pointer)
 
 (def-c-type fpos_t _G_fpos_t)
 
 (defconstant P_tmpdir "/tmp")
 
-(def-c-var stdin (:type c-pointer))
-(def-c-var stdout (:type c-pointer))
-(def-c-var stderr (:type c-pointer))
+(def-c-var stdin (:type FILE))
+(def-c-var stdout (:type FILE))
+(def-c-var stderr (:type FILE))
 
-(def-call-out clearerr (:arguments (fp c-pointer)) (:return-type nil))
-(def-call-out fclose (:arguments (fp c-pointer)) (:return-type int))
-(def-call-out feof (:arguments (fp c-pointer)) (:return-type int))
-(def-call-out ferror (:arguments (fp c-pointer)) (:return-type int))
-(def-call-out fflush (:arguments (fp c-pointer)) (:return-type int))
-(def-call-out fgetc (:arguments (fp c-pointer)) (:return-type int))
-(def-call-out fgetpos (:arguments (fp c-pointer) (pos (c-ptr fpos_t) :out))
+(def-call-out clearerr (:arguments (fp FILE)) (:return-type nil))
+(def-call-out fclose (:arguments (fp FILE)) (:return-type int))
+(def-call-out feof (:arguments (fp FILE)) (:return-type int))
+(def-call-out ferror (:arguments (fp FILE)) (:return-type int))
+(def-call-out fflush (:arguments (fp FILE)) (:return-type int))
+(def-call-out fgetc (:arguments (fp FILE)) (:return-type int))
+(def-call-out fgetpos (:arguments (fp FILE) (pos (c-ptr fpos_t) :out))
   (:return-type int))
-;(def-call-out fgets (:arguments (buf c-pointer) (size int) (fp c-pointer))
+;(def-call-out fgets (:arguments (buf c-pointer) (size int) (fp FILE))
 ;  (:return-type c-string)) ; ??
 (def-call-out fopen (:arguments (path c-string) (mode c-string))
-  (:return-type c-pointer))
+  (:return-type FILE))
 ;; it is pointless to mark (f)printf as (:built-in t)
 ;; because the signature will be different anyway (variadic!)
-(def-call-out fprintf0 (:arguments (fp c-pointer) (format c-string))
+(def-call-out fprintf0 (:arguments (fp FILE) (format c-string))
   (:return-type int) (:name "fprintf"))
-(def-call-out fprintf1i (:arguments (fp c-pointer) (format c-string) (arg int))
+(def-call-out fprintf1i (:arguments (fp FILE) (format c-string) (arg int))
   (:return-type int) (:name "fprintf"))
 (def-call-out fprintf1l
-    (:arguments (fp c-pointer) (format c-string) (arg long))
+    (:arguments (fp FILE) (format c-string) (arg long))
   (:return-type int) (:name "fprintf"))
 (def-call-out fprintf1d
-    (:arguments (fp c-pointer) (format c-string) (arg double-float))
+    (:arguments (fp FILE) (format c-string) (arg double-float))
   (:return-type int) (:name "fprintf"))
-(def-call-out fputc (:arguments (c int) (fp c-pointer))
+(def-call-out fputc (:arguments (c int) (fp FILE))
   (:return-type int))
-(def-call-out fputs (:arguments (str c-string) (fp c-pointer))
+(def-call-out fputs (:arguments (str c-string) (fp FILE))
   (:return-type int))
 ;(def-call-out fread ; ??
-;    (:arguments (ptr c-pointer) (size size_t) (nmemb size_t) (fp c-pointer))
+;    (:arguments (ptr c-pointer) (size size_t) (nmemb size_t) (fp FILE))
 ;  (:return-type size_t))
 ;(def-call-out freopen
-;    (:arguments (path c-string) (mode c-string) (fp c-pointer :in-out)) ; ??
+;    (:arguments (path c-string) (mode c-string) (fp FILE :in-out)) ; ??
 ;  (:return-type c-pointer))
-(def-call-out fscanf0 (:arguments (fp c-pointer) (format c-string))
+(def-call-out fscanf0 (:arguments (fp FILE) (format c-string))
   (:return-type int)
   (:name "fscanf"))
 (def-call-out fscanf1i
-    (:arguments (fp c-pointer) (format c-string) (arg (c-ptr int) :out))
+    (:arguments (fp FILE) (format c-string) (arg (c-ptr int) :out))
   (:return-type int) (:name "fscanf"))
 (def-call-out fscanf1l
-    (:arguments (fp c-pointer) (format c-string) (arg (c-ptr long) :out))
+    (:arguments (fp FILE) (format c-string) (arg (c-ptr long) :out))
   (:return-type int) (:name "fscanf"))
 (def-call-out fscanf1d
-    (:arguments (fp c-pointer) (format c-string)
+    (:arguments (fp FILE) (format c-string)
                 (arg (c-ptr double-float) :out))
   (:return-type int) (:name "fscanf"))
-(def-call-out fseek (:arguments (fp c-pointer) (offset long) (whence int))
+(def-call-out fseek (:arguments (fp FILE) (offset long) (whence int))
   (:return-type int))
-(def-call-out fsetpos (:arguments (fp c-pointer) (pos (c-ptr fpos_t)))
+(def-call-out fsetpos (:arguments (fp FILE) (pos (c-ptr fpos_t)))
   (:return-type int))
-(def-call-out ftell (:arguments (fp c-pointer)) (:return-type long))
+(def-call-out ftell (:arguments (fp FILE)) (:return-type long))
 ;(def-call-out fwrite ; ??
-;    (:arguments (ptr c-pointer) (size size_t) (nmemb size_t) (fp c-pointer))
+;    (:arguments (ptr c-pointer) (size size_t) (nmemb size_t) (fp FILE))
 ;  (:return-type size_t))
-(def-call-out getc (:arguments (fp c-pointer)) (:return-type int))
+(def-call-out getc (:arguments (fp FILE)) (:return-type int))
 (def-call-out getchar (:arguments) (:return-type int))
 ;(def-call-out gets (:arguments (buf c-pointer)) (:return-type c-string)); ??
 (def-call-out perror (:arguments (s c-string)) (:return-type nil))
@@ -1766,13 +1764,13 @@
   (:return-type int) (:name "printf"))
 (def-call-out printf1d (:arguments (format c-string) (arg double-float))
   (:return-type int) (:name "printf"))
-(def-call-out putc (:arguments (c int) (fp c-pointer)) (:return-type int))
+(def-call-out putc (:arguments (c int) (fp FILE)) (:return-type int))
 (def-call-out putchar (:arguments (c int)) (:return-type int))
 (def-call-out puts (:arguments (str c-string)) (:return-type int))
 (def-call-out remove (:arguments (path c-string)) (:return-type int))
 (def-call-out rename (:arguments (old c-string) (new c-string))
   (:return-type int))
-(def-call-out rewind (:arguments (fp c-pointer)) (:return-type nil))
+(def-call-out rewind (:arguments (fp FILE)) (:return-type nil))
 (def-call-out scanf0 (:arguments (format c-string))
   (:return-type int) (:name "scanf"))
 (def-call-out scanf1i (:arguments (format c-string) (arg (c-ptr int) :out))
@@ -1782,13 +1780,13 @@
 (def-call-out scanf1d
     (:arguments (format c-string) (arg (c-ptr double-float) :out))
   (:return-type int) (:name "scanf"))
-(def-call-out setbuf (:arguments (fp c-pointer) (buf c-pointer))
+(def-call-out setbuf (:arguments (fp FILE) (buf c-pointer))
   (:return-type nil))
-(def-call-out setlinebuf (:arguments (fp c-pointer)) (:return-type nil))
-(def-call-out setbuffer (:arguments (fp c-pointer) (buf c-pointer) (size int))
+(def-call-out setlinebuf (:arguments (fp FILE)) (:return-type nil))
+(def-call-out setbuffer (:arguments (fp FILE) (buf c-pointer) (size int))
   (:return-type nil))
 (def-call-out setvbuf
-    (:arguments (fp c-pointer) (buf c-pointer) (mode int) (size size_t))
+    (:arguments (fp FILE) (buf c-pointer) (mode int) (size size_t))
   (:return-type int))
 ;(def-call-out sprintf0 (:arguments (str c-pointer :out) (format c-string))
 ;  (:return-type int) (:name "sprintf")) ; ??
@@ -1811,16 +1809,16 @@
   (:return-type c-string))
 (def-call-out tempnam (:arguments (dir c-string) (prefix c-string))
   (:return-type c-string :malloc-free))
-(def-call-out ungetc (:arguments (c int) (fp c-pointer))
+(def-call-out ungetc (:arguments (c int) (fp FILE))
   (:return-type int))
 ; You can uncomment this if your compiler sets __USE_GNU
 ; (def-call-out fcloseall (:arguments) (:return-type int))
 (def-call-out fdopen (:arguments (fildes int) (mode c-string))
   (:return-type c-pointer))
-(def-call-out fileno (:arguments (fp c-pointer)) (:return-type int))
+(def-call-out fileno (:arguments (fp FILE)) (:return-type int))
 (def-call-out popen (:arguments (command c-string) (mode c-string))
   (:return-type c-pointer))
-(def-call-out pclose (:arguments (fp c-pointer)) (:return-type int))
+(def-call-out pclose (:arguments (fp FILE)) (:return-type int))
 
 (def-call-out ctermid (:arguments (null c-string)) ; ??
   (:return-type c-string))
@@ -1841,26 +1839,44 @@
 ;;; ============================== <dirent.h> ================================
 (c-lines "#include <dirent.h>~%")
 
-;;; ----------------------------- <direntry.h> -------------------------------
+;;; ----------------------------- <bits/dirent.h> ---------------------------
+;; d_type is only in dirent64, not in dirent in <linux/dirent.h>,
+;; but it appears to BE required, and does appear in <bits/dirent.h>
+
+(c-lines "#ifndef __USE_FILE_OFFSET64
+typedef __ino_t clisp_dirent_ino_t;
+typedef __off_t clisp_dirent_off_t;
+#else
+typedef __ino64_t clisp_dirent_ino_t;
+typedef __off64_t clisp_dirent_off_t;
+#endif~%")
+(def-c-type clisp_dirent_ino_t)
+(def-c-type clisp_dirent_off_t)
 
 (def-c-struct dirent
-  (d_ino long)
-  (d_off off_t)
+  (d_ino clisp_dirent_ino_t)
+  (d_off clisp_dirent_off_t)
   (d_reclen ushort)
   (d_type uchar)
-  (d_name (c-array-max character #.(cl:+ NAME_MAX 1)))
-)
+  (d_name (c-array-max character #.(cl:+ NAME_MAX 1))))
+
+(def-c-struct dirent64
+  (d_ino ino64_t)
+  (d_off off64_t)
+  (d_reclen ushort)
+  (d_type uchar)
+  (d_name (c-array-max character #.(cl:+ NAME_MAX 1))))
 
 ;;; ------------------------------ <dirent.h> --------------------------------
 
-(defconstant DT_UNKNOWN 0)
-(defconstant DT_FIFO 1)
-(defconstant DT_CHR 2)
-(defconstant DT_DIR 4)
-(defconstant DT_BLK 6)
-(defconstant DT_REG 8)
-(defconstant DT_LNK 10)
-(defconstant DT_SOCK 12)
+(def-c-const DT_UNKNOWN)
+(def-c-const DT_FIFO)
+(def-c-const DT_CHR)
+(def-c-const DT_DIR)
+(def-c-const DT_BLK)
+(def-c-const DT_REG)
+(def-c-const DT_LNK)
+(def-c-const DT_SOCK)
 
 (defmacro IFTODT (mode) `(ash (logand ,mode #o170000) -12))
 (defmacro DTTOIF (dirtype) `(ash ,dirtype 12))
@@ -1872,21 +1888,27 @@
 (def-call-out closedir (:arguments (dirp c-pointer)) (:return-type int))
 (def-call-out readdir (:arguments (dirp c-pointer))
   (:return-type (c-ptr-null dirent)))
+(def-call-out readdir64 (:arguments (dirp c-pointer))
+  (:return-type (c-ptr-null dirent64)))
 (def-call-out readdir_r
     (:arguments (dirp c-pointer) (entry (c-ptr dirent) :out :alloca)
                 (result (c-ptr (c-ptr dirent)) :out :alloca)) ; ??
   (:return-type int))
+(def-call-out readdir64_r
+    (:arguments (dirp c-pointer) (entry (c-ptr dirent64) :out :alloca)
+                (result (c-ptr (c-ptr dirent64)) :out :alloca)) ; ??
+  (:return-type int))
 
 (def-call-out rewinddir (:arguments (dirp c-pointer)) (:return-type nil))
 
 (def-call-out dirfd (:arguments (dirp c-pointer)) (:return-type int))
 
-(defconstant MAXNAMLEN 255)
+(def-c-const MAXNAMLEN)
 
-(def-call-out seekdir (:arguments (dirp c-pointer) (pos off_t))
+(def-call-out seekdir (:arguments (dirp c-pointer) (pos long))
   (:return-type nil))
 
-(def-call-out telldir (:arguments (dirp c-pointer)) (:return-type off_t))
+(def-call-out telldir (:arguments (dirp c-pointer)) (:return-type long))
 
 (def-call-out scandir
     (:arguments (dir c-string)
@@ -1898,10 +1920,29 @@
                                                 (d2 (c-ptr (c-ptr dirent))))
                                     (:return-type int))))
   (:return-type int))
+(def-call-out scandir64
+    (:arguments (dir c-string)
+                (namelist (c-ptr (c-ptr (c-ptr dirent64))) :out)
+                ;; select may also be NULL/NIL, which c-function accepts
+                (select (c-function (:arguments (d c-string))
+                                    (:return-type boolean)))
+                (compar (c-function (:arguments (d1 (c-ptr (c-ptr dirent64)))
+                                                (d2 (c-ptr (c-ptr dirent64))))
+                                    (:return-type int))))
+  (:return-type int))
 
 (def-call-out alphasort
     (:arguments (d1 (c-ptr (c-ptr dirent))) (d2 (c-ptr (c-ptr dirent))))
   (:return-type int))
+(def-call-out alphasort64
+    (:arguments (d1 (c-ptr (c-ptr dirent64))) (d2 (c-ptr (c-ptr dirent64))))
+  (:return-type int))
+(def-call-out versionsort
+    (:arguments (d1 (c-ptr (c-ptr dirent))) (d2 (c-ptr (c-ptr dirent))))
+  (:return-type int))
+(def-call-out versionsort64
+    (:arguments (d1 (c-ptr (c-ptr dirent64))) (d2 (c-ptr (c-ptr dirent64))))
+  (:return-type int))
 
 ;(def-call-out getdirentries
 ;    (:arguments (fd int) (buf c-pointer) (nbytes size_t)
@@ -2026,9 +2067,9 @@
 
 ;;; ----------------------------- <termbits.h> -------------------------------
 
-(def-c-type cc_t uchar)
-(def-c-type speed_t uint)
-(def-c-type tcflag_t uint)
+(def-c-type cc_t)               ; uchar
+(def-c-type speed_t)            ; uint
+(def-c-type tcflag_t)           ; uint
 
 (eval-when (load compile eval)
   (defconstant NCCS 32)
@@ -2355,11 +2396,11 @@ and SIG_IGN do anyway, by other means ... They are just sugar, really.
 ;;; --------------------------------- <bits/sigset.h> -----------------------
 (eval-when (load compile eval)
   (defconstant SIGSET_NWORDS
-    ;; #.(cl:/ 1024 #.(cl:* 8 (ffi:sizeof 'ffi:uint)))
-    32))
+    ;; 32 on 32-bit platforms, 16 on 64-bit platforms
+    #.(cl:/ 1024 #.(cl:* 8 (ffi:sizeof 'ffi:ulong)))))
 
 (def-c-struct sigset_t
-  (val (c-array-max uint #.SIGSET_NWORDS)))
+  (val (c-array-max ulong #.SIGSET_NWORDS)))
 ;;; --------------------------------- <bits/sigaction.h> ---------------------
 
 (def-c-type sighandler_t (c-function (:arguments (sig int)) ;from signal.h
@@ -2368,8 +2409,19 @@ and SIG_IGN do anyway, by other means ... They are just sugar, really.
 (def-c-struct sigaction
   (sa_handler sighandler_t)
   (sa_mask sigset_t)
-  (sa_flags uint32) ; actually int but otherwise CLISP can't coerce SA_RESETHAND?!
-  (sa_restorer (c-function (:arguments) (:return-type nil))))
+  (sa_flags uint32) ; actually int but otherwise CLISP can't coerce SA_RESETHAND
+  ;; The sa_restorer element is obsolete and should not be used.
+  ;; POSIX does not specify a sa_restorer element.
+  (sa_restorer c-pointer))      ; (c-function (:arguments) (:return-type nil))
+
+#+(or)(progn ;; DEBUG
+(def-c-const sigaction-size (:name "sizeof(struct sigaction)") (:guard nil))
+(def-c-const sigset-size (:name "sizeof(sigset_t)") (:guard nil))
+(def-c-const _SIGSET_NWORDS)
+(assert (= (sizeof 'sigaction) sigaction-size))
+(assert (= (sizeof 'sigset_t) sigset-size))
+(assert (= SIGSET_NWORDS _SIGSET_NWORDS))
+)
 
 (defconstant SA_NOCLDSTOP  1)   ; Don't send SIGCHLD when children stop.
 (defconstant SA_NOCLDWAIT  2)   ; Don't create zombie on child death.
