@@ -3,7 +3,7 @@
  * Fred Cohen, 2003-2004
  * Don Cohen, 2003-2004
  * Sam Steingold 2004-2008
- * Bruno Haible 2004-2005
+ * Bruno Haible 2004-2008
  * <http://www.opengroup.org/onlinepubs/007908799/xns/syssocket.h.html>
  */
 
@@ -15,25 +15,21 @@
 
 #include "config.h"
 
-#if defined(TIME_WITH_SYS_TIME)
+#if defined(HAVE_SYS_TIME_H)
 # include <sys/time.h>
+#endif
+#if defined(HAVE_TIME_H)
 # include <time.h>
-#else
-# if defined(HAVE_SYS_TIME_H)
-#  include <sys/time.h>
-# elif defined(HAVE_TIME_H)
-#  include <time.h>
-# endif
 #endif
 #if defined(HAVE_SYS_TYPES_H)
 # include <sys/types.h>
 #endif
-#if defined(STDC_HEADERS)
-# include <stdio.h>
+#include <stdio.h>
+#if defined(HAVE_UNISTD_H)
 # include <unistd.h>
-# include <string.h>            /* for memcpy(3) */
-# include <stddef.h>            /* for offsetof */
 #endif
+#include <string.h>            /* for memcpy(3) */
+#include <stddef.h>            /* for offsetof */
 #if defined(HAVE_SYS_SOCKET_H)
 # include <sys/socket.h>
 #endif
@@ -498,9 +494,9 @@ DEFUN(RAWSOCK:NETWORK, &optional network type)
     VALUES1(listof(count));
     return;
   } else if (uint_p(net)) {
-#  if defined(HAVE_GETNETBYNUMBER)
+#  if defined(HAVE_GETNETBYADDR)
     begin_system_call();
-    ne = getnetbynumber(I_to_uint(net),type);
+    ne = getnetbyaddr(I_to_uint(net),type);
     end_system_call();
 #  endif
   } else if (stringp(net)) {
@@ -971,7 +967,7 @@ static void check_message (gcv_object_t *mho, uintL *offset, struct msghdr *mhp)
   TheStructure(*mho)->recdata[MSG_CONTROL] =
     check_byte_vector(TheStructure(*mho)->recdata[MSG_CONTROL]);
   mhp->msg_flags =
-    check_msg_flags_from_list(TheStructure(*mho)->recdata[MSG_FLAGS]);
+    check_msg_flags_of_list(TheStructure(*mho)->recdata[MSG_FLAGS]);
 }
 /* fill msg_controllen, msg_control, msg_iov, msg_name from mho */
 static void fill_msghdr (gcv_object_t *mho, uintL offset, struct msghdr *mhp,
@@ -1084,7 +1080,7 @@ DEFUN(RAWSOCK:SOCK-WRITE,socket buffer &key :START :END)
   ssize_t retval;
   size_t len;
   uintL offset;
-  if ((retval = check_iovec_arg(&STACK_2,&offset)) >= 0) { /* WRITEW */
+  if ((retval = check_iovec_arg(&STACK_2,&offset)) >= 0) { /* WRITEV */
     struct iovec *buffer = (struct iovec*)alloca(sizeof(struct iovec)*retval);
     fill_iovec(STACK_0,offset,retval,buffer,PROT_READ);
     SYSCALL(retval,sock,writev(sock,buffer,retval));
