@@ -1,5 +1,5 @@
 ;;; handle the posix functions
-;;; Sam Steingold 1999-2007
+;;; Sam Steingold 1999-2008
 
 (defpackage #:posix
   (:use #:common-lisp #:ext)
@@ -11,9 +11,10 @@
    #:stream-lock #:with-stream-lock #:duplicate-handle #:copy-file
    #:file-owner #:physical-memory #:stream-options #+unix #:string-time
    #+(or :win32 :cygwin) #:file-properties #+unix #:make-xterm-io-stream
-   #:priority #:process-id #:openlog #:setlogmask #:syslog #:closelog
-   #:getpgid #:setpgrp #:getsid #:setsid #:setpgid #:kill #:sync
-   #:erf #:erfc #:j0 #:j1 #:jn #:y0 #:y1 #:yn #:gamma #:lgamma #:ffs))
+   #:priority #:openlog #:setlogmask #:syslog #:closelog #:process-id #:getppid
+   #:getsid #:setsid #:getpgrp #:setpgrp #:setreuid #:setregid #:kill #:sync
+   #:errno #:strerror #:gethostid
+   #:erf #:erfc #:j0 #:j1 #:jn #:y0 #:y1 #:yn #:tgamma #:lgamma #:ffs))
 
 (pushnew :syscalls *features*)
 (provide "syscalls")
@@ -29,11 +30,13 @@
 )
 ;;; ============================================================
 #+unix (progn
-(export '(getuid getgid geteuid getegid))
-(defsetf getuid posix::%setuid)
-(defsetf getgid posix::%setgid)
-(defsetf geteuid posix::%seteuid)
-(defsetf getegid posix::%setegid)
+(export '(uid gid euid egid pgid groups))
+(defsetf uid posix::%setuid)
+(defsetf gid posix::%setgid)
+(defsetf euid posix::%seteuid)
+(defsetf egid posix::%setegid)
+(defsetf pgid posix::%setpgid)
+(defsetf groups posix::%setgroups)
 )
 ;;; ============================================================
 (defmacro with-stream-lock ((stream &rest options) &body body)
@@ -50,7 +53,7 @@
   `(%syslog ,severity ,facility (format nil ,format ,@args)))
 ;;; ============================================================
 (defsetf priority (pid &optional which) (value)
-  `(set-priority ,value ,pid ,which))
+  `(%set-priority ,value ,pid ,which))
 ;;; ============================================================
 (defstruct (hostent (:constructor
                      make-hostent (name aliases addr-list addrtype)))
@@ -266,7 +269,9 @@
                    (* page-size (sysconf :AVPHYS-PAGES))))
   #+win32 (let ((mem-stat (memory-status)))
             (values (memstat-total-physical mem-stat)
-                    (memstat-avail-physical mem-stat))))
+                    (memstat-avail-physical mem-stat)))
+  #-(or unix win32)
+  (error "~S: only ~S and ~S are supported" 'physical-memory :unix :win32))
 
 ;;;--------------------------------------------------------------------------
 ;;; service / port map
@@ -281,7 +286,9 @@
 (defvar *services*)
 (defvar *services-file*
   #+unix "/etc/services"
-  #+win32 (string-concat (getenv "WINDIR") "/system32/drivers/etc/services"))
+  #+win32 (string-concat (getenv "WINDIR") "/system32/drivers/etc/services")
+  #-(or unix win32)
+  (error "~S: only ~S and ~S are supported" '*services-file* :unix :win32))
 (defun service (&optional service-name protocol)
   (unless (boundp '*services*)
     (setq *services*
@@ -355,19 +362,18 @@
 
 ;;;--------------------------------------------------------------------------
 #+unix
-(defun make-xterm-io-stream (&key (title "CLISP I/O"))
-  (let* ((tmps (mkstemp "/tmp/clisp-x-io-XXXXXX"))
-         (pipe (namestring tmps))
+(defun make-xterm-io-stream (&key (title "CLISP I/O") (xterm "xterm"))
+  (let* ((tmpdir (mkdtemp "/tmp/clisp-x-io-XXXXXX/"))
+         (pipe (namestring (merge-pathnames "pipe" tmpdir)))
          xio
          (clos::*enable-clos-warnings* nil))
-    (close tmps) (delete-file tmps)
-    (mknod tmps :FIFO :RWXU)
+    (mknod pipe :FIFO :RWXU)
     (setq title (string title))
     ;; - tty tells us what device to use for IO
     ;; - cat holds xterm from quitting and prints the good-bye message
     ;;   (actually, the xterm window disappears before you can read it...)
-    (shell (format nil "xterm -n ~s -T ~s -e 'tty >> ~a; cat ~a' &"
-                   title title pipe pipe))
+    (shell (format nil "~a -n ~s -T ~s -e 'tty >> ~a; cat ~a' &"
+                   xterm title title pipe pipe))
     (with-open-file (s pipe :direction :input)
       (setq xio (open (read-line s) :direction :io)))
     ;; GC uses the internal non-generic builtin_stream_close() so
@@ -381,10 +387,18 @@
       (with-open-file (s pipe :direction :output)
         (write-line (SYS::TEXT "Bye.") s))
       (delete-file pipe)
+      (delete-directory tmpdir)
       (let ((clos::*enable-clos-warnings* nil))
         (remove-method #'close (find-method #'close '(:after) `((eql ,xio))))))
     xio))
-
+;;;--------------------------------------------------------------------------
+#+FFI (progn
+(export '(fopen fdopen freopen fclose fflush ; fgetc fputc ungetc
+          clearerr feof ferror fileno stdin stdout stderr))
+(defconstant stdin (posix::%stdio 0))
+(defconstant stdout (posix::%stdio 1))
+(defconstant stderr (posix::%stdio 2))
+)
 ;;;--------------------------------------------------------------------------
 (defun ffs (n) (integer-length (logand n (- n))))
 ;; http://www.opengroup.org/onlinepubs/009695399/functions/ffs.html
@@ -392,6 +406,11 @@
 ;; (ffi:def-call-out ffs (:name "ffs") (:arguments (i ffi:int))
 ;;   (:return-type ffi:int) (:language :stdc) (:library :default))
 ;; and it also supports bignums.
+(unless (fboundp 'os:tgamma)
+  ;; https://sourceforge.net/tracker/?func=detail&atid=101355&aid=1966375&group_id=1355
+  (defun os:tgamma (x)
+    (multiple-value-bind (lg sg) (os:lgamma x)
+      (* (or sg 1) (exp lg)))))
 ;;;--------------------------------------------------------------------------
 (setf (package-lock "EXT") nil)
 (use-package '("POSIX") "EXT")
