@@ -1,6 +1,6 @@
 ;;; ANSI-compatible definitions + some extensions
 ;;; Bruno Haible 21.7.1994 - 2006
-;;; Sam Steingold 1999-2004, 2007
+;;; Sam Steingold 1999-2004, 2007-2008
 
 ;; ============================================================================
 
@@ -111,7 +111,7 @@
            (*PRINT-LEVEL*               NIL)
            (*PRINT-LINES*               NIL)
            (*PRINT-MISER-WIDTH*         NIL)
-           (*PRINT-PPRINT-DISPATCH*     NIL)
+           (*PRINT-PPRINT-DISPATCH*     (COPY-PPRINT-DISPATCH NIL))
            (*PRINT-PRETTY*              NIL)
            (*PRINT-RADIX*               NIL)
            (*PRINT-READABLY*            T)
@@ -169,7 +169,7 @@
                     (unless nextch
                       (error-of-type 'end-of-file
                         :stream stream
-                        (TEXT "~S: input stream ~S ends within read macro beginning to ~S")
+                        (TEXT "~S: input stream ~S ends within read macro beginning with ~S")
                         'read stream ch
                     ) )
                     (unless (characterp nextch)
@@ -206,7 +206,7 @@
   (let ((vector-index
           (do ((i 0 (1+ i)))
                (nil)
-            (when (eq (%record-ref #'dispatch-reader i) vector) (return i)))))
+            (when (eq (closure-const #'dispatch-reader i) vector) (return i)))))
     (%defio #'dispatch-reader vector-index)
   )
 )
@@ -242,7 +242,7 @@
       (T (return))))
   (stream-element-type stream))
 
-(defun read-sequence (sequence stream &rest rest &key (start 0) (end nil))
+(defun %read-sequence (sequence stream &rest rest &key (start 0) (end nil))
   (declare (ignore start end))
   (let ((seltype (stream-input-element-type stream))
         (veltype (if (vectorp sequence) (array-element-type sequence) t)))
@@ -256,7 +256,13 @@
                   'read-sequence sequence stream
                   'read-char-sequence 'read-byte-sequence)))))
 
-(defun write-sequence (sequence stream &rest rest &key (start 0) (end nil))
+(defun read-sequence (sequence stream &rest rest &key (start 0) (end nil))
+  (declare (ignore start end))
+  (if (built-in-stream-p stream)
+      (apply #'%read-sequence sequence stream rest)
+      (apply 'gray::stream-read-sequence sequence stream rest)))
+
+(defun %write-sequence (sequence stream &rest rest &key (start 0) (end nil))
   (declare (ignore start end))
   (let ((seltype (stream-output-element-type stream))
         (veltype (if (vectorp sequence) (array-element-type sequence) t)))
@@ -273,6 +279,12 @@
                   'write-sequence sequence stream
                   'write-char-sequence 'write-byte-sequence)))))
 
+(defun write-sequence (sequence stream &rest rest &key (start 0) (end nil))
+  (declare (ignore start end))
+  (if (built-in-stream-p stream)
+      (apply #'%write-sequence sequence stream rest)
+      (apply 'gray::stream-write-sequence sequence stream rest)))
+
 ;; ----------------------------------------------------------------------------
 
 ;; ANSI-CL specifies TYPE-ERRORs in many places.
@@ -405,13 +417,7 @@
 ;; ----------------------------------------------------------------------------
 
 (defmacro define-hash-table-test (name test hash)
-  (loop (when (symbolp name) (return))
-    ;; check-value will be defined in condition.lisp
-    (setq name (check-value
-                nil (make-condition 'simple-type-error
-                      :format-control (TEXT "~S: ~S should be a symbol")
-                      :format-arguments (list 'define-hash-table-test name)
-                      :datum name :expected-type 'symbol))))
+  (setq name (check-symbol name 'define-hash-table-test))
   `(progn (setf (get ',name 'hash-table-test) (cons #',test #',hash)) ',name))
 
 ;; (default-directory) is a Synonym for (cd).
