@@ -1,6 +1,6 @@
 ;;; Foreign function interface for CLISP
 ;;; Bruno Haible 19.2.1995
-;;; Sam Steingold 1998-2006
+;;; Sam Steingold 1998-2008
 
 #+UNICODE
 (progn
@@ -12,7 +12,7 @@
 (in-package "FFI")
 
 (export '(def-c-type def-c-var def-c-const parse-c-type deparse-c-type
-          def-c-call-out def-call-out #+AFFI def-lib-call-out
+          def-c-call-out def-call-out
           def-c-call-in def-call-in default-foreign-language
           c-lines *output-c-functions* *output-c-variables* *foreign-guard*
           nil boolean character char uchar short ushort int uint long ulong
@@ -20,14 +20,15 @@
           single-float double-float default-foreign-library
           c-pointer c-string c-struct c-union c-array c-array-max
           c-function c-ptr c-ptr-null c-array-ptr
+          size_t ssize_t |file|
           def-c-enum def-c-struct element deref slot cast typeof
           sizeof bitsizeof c-var-object c-var-address offset
           validp with-c-place foreign-value enum-from-value enum-to-value
           foreign-address foreign-address-unsigned unsigned-foreign-address
           with-foreign-object with-c-var with-foreign-string
           foreign-allocate allocate-deep allocate-shallow foreign-free
-          foreign-pointer set-foreign-pointer
-          close-foreign-library memory-as
+          foreign-pointer set-foreign-pointer foreign-pointer-info
+          open-foreign-library close-foreign-library memory-as
           foreign-variable foreign-function))
 
 (eval-when (load compile eval)
@@ -123,8 +124,11 @@
           '(nil boolean character char uchar short ushort int uint long ulong
             uint8 sint8 uint16 sint16 uint32 sint32 uint64 sint64
             single-float double-float
+            size_t ssize_t      ; see foreign.d:init_ffi()
             c-pointer c-string))
   (setf (gethash c-type *c-type-table*) c-type))
+;; FILE* is used by modern ffi packages
+(setf (gethash '|file| *c-type-table*) 'c-pointer) ; (def-c-type FILE c-pointer)
 
 ;; parse the components of a C-STRUCT or C-UNION
 (defun parse-components (typespec)
@@ -373,7 +377,7 @@
     (coerce (mapcap (lambda (argspec)
                       (unless (and (listp argspec)
                                    (symbolp (first argspec))
-                                   (<= 2 (length argspec) #-AFFI 4 #+AFFI 5))
+                                   (<= 2 (length argspec) 4))
                         (error (TEXT "Invalid parameter specification in ~S: ~S")
                                whole argspec))
                       (let* ((argtype (parse-c-type (second argspec)))
@@ -388,26 +392,22 @@
                         ;; see FOREIGN-CALL-OUT in foreign.d
                         (when (and (or (eq argmode :OUT) (eq argmode :IN-OUT))
                                    (not (eq (ctype-type argtype) 'C-PTR)))
-                          (warn (TEXT "~s argument ~s is not a pointer in ~s")
+                          (warn (TEXT "~S argument ~S is not a pointer in ~S")
                                 argmode argtype whole))
                         (list argtype
-                              (+ (ecase argmode
+                              (+ (sys::mecase whole argmode
                                    ((:IN :READ-ONLY) 0)
                                    ((:OUT :WRITE-ONLY) ff-flag-out)
                                    ((:IN-OUT :READ-WRITE) ff-flag-in-out))
-                                 (ecase argalloc
+                                 (sys::mecase whole argalloc
                                    (:NONE 0)
                                    (:ALLOCA ff-flag-alloca)
-                                   (:MALLOC-FREE ff-flag-malloc-free))
-                                 #+AFFI
-                                 (if (cddddr argspec)
-                                   (ash (1+ (position (fifth argspec) *registers*)) 8)
-                                   0)))))
+                                   (:MALLOC-FREE ff-flag-malloc-free))))))
                     (or (rest (assoc ':arguments alist)) '()))
             'simple-vector)
     (+ (let ((rettype (assoc ':return-type alist)))
          (if (cddr rettype)
-           (ecase (third rettype)
+           (sys::mecase whole (third rettype)
              (:NONE 0)
              (:MALLOC-FREE ff-flag-malloc-free))
            0))
@@ -417,7 +417,7 @@
            (language-to-flag
             (or *foreign-language*
                 (progn
-                  (warn (TEXT "~s: No ~s argument and no ~s form in this compilation unit; ~s assumed now and for the rest of this unit")
+                  (warn (TEXT "~S: No ~S argument and no ~S form in this compilation unit; ~S assumed now and for the rest of this unit")
                         whole :language 'default-foreign-language :stdc)
                   (setq *foreign-language* :STDC))))))))) ; Default is ANSI C
 
@@ -430,10 +430,14 @@
     (error (TEXT "The name ~S is not a valid C identifier")
            name)))
 
-(defmacro DEF-C-TYPE (&whole whole-form name typespec)
+(defmacro DEF-C-TYPE (&whole whole-form name &optional typespec)
   (setq name (check-symbol name (first whole-form)))
   `(EVAL-WHEN (LOAD COMPILE EVAL)
-     (PARSE-C-TYPE ',typespec ',name)
+     ,(if typespec
+          `(PARSE-C-TYPE ',typespec ',name)
+          `(SETF (GETHASH ',name FFI::*C-TYPE-TABLE*)
+                 ;; prefer lower case
+                 (CS-CL:symbol-name ',name)))
      ',name))
 
 ;; Convert back a C type from internal (vector) to external (list)
@@ -500,12 +504,7 @@
                                                                (t ':IN))
                                                         ,(cond ((flag-set-p argflags ff-flag-alloca) ':ALLOCA)
                                                                ((flag-set-p argflags ff-flag-malloc-free) ':MALLOC-FREE)
-                                                               (t ':NONE))
-                                                        #+AFFI
-                                                        ,@(let ((h (logand (ash argflags -8) #xF)))
-                                                            (if (not (zerop h))
-                                                              (list (svref *registers* (- h 1)))
-                                                              '())))
+                                                               (t ':NONE)))
                                                       argspecs))))
                                       (list ':return-type
                                             (deparse (svref ctype 1))
@@ -719,19 +718,22 @@
               *c-name* count))
     (format *coutput-stream* "~%")
     (maphash (lambda (type fun-vec)
-               (let* ((fun (car fun-vec)) (vec (cdr fun-vec))
+               (let* ((fun (first fun-vec)) (vec (second fun-vec))
                       (c-decl (to-c-typedecl type fun)))
                  (format *coutput-stream* "~A (int number, int *definedp);~%~
                                            ~A (int number, int *definedp) {
   *definedp=1;~%  switch (number) {~%"
                          c-decl c-decl)
                  (dotimes (num (length vec))
-                   (let ((const (aref vec num)))
-                     (format *coutput-stream* "#  if defined(~A)
-    case ~D: return ~A;~%#  endif~%"
-                             const num const)))
+                   (destructuring-bind (const . guard) (aref vec num)
+                     (when guard
+                       (format *coutput-stream* "#  if ~A~%" guard))
+                     (format *coutput-stream* "    case ~D: return ~A;~%"
+                             num const)
+                     (when guard
+                       (format *coutput-stream* "#  endif~%"))))
                  (format *coutput-stream* "    default: *definedp=0; return 0;
-  }~%}~%#define HAVE_~A~%" (string-upcase (symbol-name fun)))))
+  }~%}~%")))
              *constant-table*)
     (setq *variable-list*
           (nreverse (delete-duplicates
@@ -765,11 +767,23 @@
             ~%void module__~A__init_function_2 (module_t* module);~%~
             ~%void module__~A__fini_function (module_t* module);~%~
             ~2%void module__~A__init_function_1 (module_t* module)~%~
-            {~{~%~A~}~%}~2%~
-            void module__~A__init_function_2 (module_t* module)~%~
             {~{~%~A~}~%"
-            *c-name* *c-name* *c-name*
-            *c-name* *init-once* *c-name* *init-always*)
+            *c-name* *c-name* *c-name* *c-name* *init-once*)
+    (let ((done (make-hash-table :test 'equal)))
+      (maphash (lambda (type spec)
+                 (declare (ignore type))
+                 (when (and (stringp spec) (not (gethash spec done)))
+                   (setf (gethash spec done) spec)
+                   (when *foreign-guard*
+                     (format *coutput-stream* "# if HAVE_~A~%"
+                             (string-upcase spec)))
+                   (format *coutput-stream* "  register_foreign_inttype(~S,sizeof(~A),(~A)-1<=(~A)0);~%" spec spec spec spec)
+                   (when *foreign-guard* (format *coutput-stream* "# endif~%"))))
+               *c-type-table*))
+    (format *coutput-stream*
+            "}~2%void module__~A__init_function_2 (module_t* module)~%~
+            {~{~%~A~}~%"
+             *c-name* *init-always*)
     (dolist (variable *variable-list*)
       (let ((c-name (first variable)))
         (when *foreign-guard*
@@ -788,6 +802,13 @@
                 "  register_foreign_function((void*)&~A,~A,~D);~%"
                 c-name (to-c-string c-name) (svref (second function) 3))
         (when *foreign-guard* (format *coutput-stream* "# endif~%"))))
+    (maphash (lambda (type fun-vec)
+               (declare (ignore type))
+               (let ((c-name (to-c-name (car fun-vec))))
+                 (format *coutput-stream*
+                         "  register_foreign_function((void*)&~A,\"~A\",~D);~%"
+                         c-name c-name (svref (third fun-vec) 3))))
+             *constant-table*)
     (format *coutput-stream*
             "}~2%void module__~A__fini_function (module_t* module)~%~
             {~{~%~A~}~%}~%"
@@ -838,6 +859,10 @@
     (parse-foreign-name (second name-option))
     (to-c-name lisp-name)))
 
+(defun get-assoc (key alist default)
+  (let ((pair (assoc key alist)))
+    (if pair (second pair) default)))
+
 ;; CPP consts (#define'd in an *.h file):
 ;; for each type (int, string, pointer) there is a C function (and a
 ;;  foreign-function created when the first constant of this type is
@@ -846,35 +871,52 @@
 ;;  into the C file and compiled, so the numbers have to be pre-assigned
 (defmacro DEF-C-CONST (&whole whole-form name &rest options)
   (setq name (check-symbol name (first whole-form)))
-  (prepare-module)
-  (let* ((alist (parse-options options '(:name :type :documentation)
+  (let* ((alist (parse-options options '(:name :type :documentation :guard)
                                whole-form))
          (doc (cdr (assoc ':documentation alist))) ; ("doc string") or NIL
-         (c-type (or (second (assoc ':type alist)) 'ffi:int))
-         (c-name (foreign-name name (assoc ':name alist)))
-         f-name c-number)
+         (c-type (get-assoc :type alist 'ffi:int))
+         (c-name (get-assoc :name alist name))
+         (guard (get-assoc :guard alist (format nil "defined(~A)" c-name)))
+         (cftype (parse-c-function
+                  `((:arguments (number int) (defined-p (c-ptr int) :out))
+                    (:return-type ,c-type))
+                  whole-form)))
     (check-type c-type (member ffi:int ffi:c-string ffi:c-pointer)
-                "A constant must be either an integer, a string or a pointer")
-    (setq f-name (intern
-                  (format nil "module__~A__constant_map_~A" *name*
-                          (nstring-downcase
-                           (nsubstitute #\_ #\-
-                                        (copy-seq (symbol-name c-type))))))
-          c-number
-          (vector-push-extend
-           c-name (cdr (or (gethash c-type *constant-table*)
-                           (setf (gethash c-type *constant-table*)
-                                 (cons f-name (make-array 10 :adjustable t
-                                                          :fill-pointer 0)))))))
-    `(progn
-       ;; c-number == 0 ==> need to output the def-call-out form
-       ,@(when (zerop c-number)
-           `((ffi:def-call-out ,f-name
-                 (:arguments (number ffi:int)
-                             (defined-p (ffi:c-ptr ffi:int) :out))
-               (:return-type ,c-type))))
-       (defconstant ,name (c-const-value ',f-name ,c-number ',name ',c-name)
-         ,@doc))))
+                "an integer, a string or a pointer")
+    `(let ((f-name&c-number
+            (compile-time-value (note-c-const ',c-name ',c-type
+                                              ',cftype ',guard))))
+       (unless f-name&c-number
+         (error (TEXT "~S(~S ~S) requires writing to a C file")
+                'def-c-const ',name ',c-name))
+       (let ((f-name (first f-name&c-number))
+             (c-number (second f-name&c-number)))
+         ;; c-number == 0 ==> need to output the def-call-out form
+         (when (zerop c-number)
+           (SYSTEM::%PUTD
+            ;; we do not really need to name this foreign function; this is just
+            ;; to avoid an HT lookup in FIND-FOREIGN-FUNCTION for each C const
+            f-name (FIND-FOREIGN-FUNCTION (to-c-name f-name)
+                                          ,cftype NIL NIL NIL NIL)))
+         (defconstant ,name (c-const-value f-name c-number ',name ',c-name)
+           ,@doc)))))
+
+(defun note-c-const (c-name c-type cftype guard)
+  (when (compiler::prepare-coutput-file)
+    (prepare-module)
+    (let ((f-name (intern
+                   (format nil "module__~A__constant_map_~A" *name*
+                           (nstring-downcase
+                            (nsubstitute #\_ #\-
+                                         (copy-seq (symbol-name c-type))))))))
+      (list f-name
+            (vector-push-extend
+             (cons c-name guard)
+             (second (or (gethash c-type *constant-table*)
+                         (setf (gethash c-type *constant-table*)
+                               (list f-name (make-array 10 :adjustable t
+                                                        :fill-pointer 0)
+                                     cftype)))))))))
 
 (defun c-const-value (f-name c-number name c-name) ; ABI
   (multiple-value-bind (value value-p) (funcall f-name c-number)
@@ -884,15 +926,11 @@
                 'def-c-const name c-name)
           (sys::%unbound)))))
 
-(defun get-assoc (key alist default)
-  (let ((pair (assoc key alist)))
-    (if pair (second pair) default)))
-
 (defmacro DEF-C-VAR (&whole whole-form
                      name &rest options)
   (setq name (check-symbol name (first whole-form)))
   (let* ((alist (parse-options options '(:name :type :read-only :alloc
-                                         :library :documentation)
+                                         :library :version :documentation)
                                whole-form))
          (doc (assoc ':documentation alist))
          (c-name (foreign-name name (assoc ':name alist)))
@@ -906,11 +944,12 @@
          (flags (+ (if read-only fv-flag-readonly 0)
                    (let ((alloc (assoc ':alloc alist)))
                      (if (cdr alloc)
-                       (ecase (second alloc)
+                       (sys::mecase whole-form (second alloc)
                          (:NONE 0)
                          (:MALLOC-FREE fv-flag-malloc-free))
                        0))))
          (library (get-assoc :library alist '*foreign-library*))
+         (version (second (assoc :version alist)))
          #|
          (getter-function-name (sys::symbol-suffix name "%GETTER%"))
          (setter-function-name (sys::symbol-suffix name "%SETTER%"))
@@ -918,7 +957,7 @@
          (def (gensym "DEF-C-VAR-")))
     `(LET ((,def (LOAD-TIME-VALUE
                   (FIND-FOREIGN-VARIABLE
-                   ',c-name (PARSE-C-TYPE ',type) ,library NIL))))
+                   ',c-name (PARSE-C-TYPE ',type) ,library ,version NIL))))
        #|
        (LET ((FVAR (LOOKUP-FOREIGN-VARIABLE ',c-name (PARSE-C-TYPE ',type))))
          (DEFUN ,getter-function-name () (FOREIGN-VALUE FVAR))
@@ -1022,7 +1061,7 @@
 ;; ============================ named C functions ============================
 
 (defmacro DEF-C-CALL-OUT (name &rest options)
-  (warn (TEXT "~s is deprecated, use ~s instead")
+  (warn (TEXT "~S is deprecated, use ~S instead")
         'def-c-call-out 'def-call-out)
   `(DEF-CALL-OUT ,name ,@options (:LANGUAGE :STDC)))
 
@@ -1030,7 +1069,7 @@
   (setq name (check-symbol name (first whole-form)))
   (let* ((alist
           (parse-options options '(:name :arguments :return-type :language
-                                   :built-in :library :documentation)
+                                   :built-in :library :version :documentation)
                          whole-form))
          (def (gensym "DEF-CALL-OUT-"))
          (properties (and (>= 1 (compiler::declared-optimize
@@ -1038,13 +1077,14 @@
                                              system::*denv*)))
                           (assoc ':documentation alist)))
          (library (get-assoc :library alist '*foreign-library*))
+         (version (second (assoc :version alist)))
          (c-name (foreign-name name (assoc :name alist)))
          (built-in (second (assoc :built-in alist)))
          ;; Maximize sharing in .fas file, reuse options
          ;; parse-c-function ignores unknown options, e.g. :name
          (ctype `(PARSE-C-FUNCTION ',options ',whole-form)))
     `(LET ((,def (FIND-FOREIGN-FUNCTION
-                  ',c-name ,ctype ',properties ,library NIL)))
+                  ',c-name ,ctype ',properties ,library ,version NIL)))
        (EXT:COMPILER-LET ((,def ,ctype))
          (EVAL-WHEN (COMPILE)
            (UNLESS ,LIBRARY (NOTE-C-FUN ',c-name ,def ',built-in)))
@@ -1061,27 +1101,6 @@
     (push (list c-name ctype built-in)
           *function-list*)))
 
-#+AFFI
-(defmacro DEF-LIB-CALL-OUT (&whole whole-form name library &rest options)
-  (setq name (check-symbol name (first whole-form)))
-  (let* ((alist (parse-options options
-                               '(:name :offset :arguments :return-type)
-                               whole-form))
-         (properties (and (>= 1 (compiler::declared-optimize
-                                 'space (and (boundp 'system::*denv*)
-                                             system::*denv*)))
-                          (assoc ':documentation alist)))
-         (c-name (foreign-name name (assoc ':name alist)))
-         (offset (second (assoc ':offset alist)))
-         (ctype `(PARSE-C-FUNCTION ',options ',whole-form)))
-    `(LET ()
-       (SYSTEM::REMOVE-OLD-DEFINITIONS ',name)
-       (COMPILER::EVAL-WHEN-COMPILE
-         (COMPILER::C-DEFUN ',name (C-TYPE-TO-SIGNATURE ,ctype)))
-       (SYSTEM::%PUTD ',name (FIND-FOREIGN-FUNCTION
-                              ',c-name ,ctype ',properties ',library ',offset))
-       ',name)))
-
 (defun count-inarguments (arg-vector)
   (do* ((l (length arg-vector))
         (inargcount 0)
@@ -1096,10 +1115,10 @@
 
 ; Called by SYS::FUNCTION-SIGNATURE.
 (defun foreign-function-in-arg-count (obj)
-  (count-inarguments (sys::%record-ref obj 3)))
+  (count-inarguments (sys::%record-ref obj 4))) ; ff_argtypes
 
 (defmacro DEF-C-CALL-IN (name &rest options)
-  (warn (TEXT "~s is deprecated, use ~s instead")
+  (warn (TEXT "~S is deprecated, use ~S instead")
         'def-c-call-in 'def-call-in)
   `(DEF-CALL-IN ,name ,@options (:LANGUAGE :STDC)))
 
