@@ -83,16 +83,25 @@
   "The final part of the prompt")
 (defun prompt-finish () (prompt-to-string *prompt-finish*))
 
+;; http://sourceforge.net/tracker/index.php?func=detail&aid=1865636&group_id=1355&atid=101355
+;; [ 1865636 ] infinite recursive error reporting
+(defun safe-wr-st (string &optional (stream *standard-output*))
+  (handler-case (write-string string stream)
+    (system::charset-type-error ()
+      (format stream "Unprintable message: change ~S or ~S" '*current-language*
+              '*terminal-encoding*))))
+
 ;; Help-function:
 (defvar *key-bindings* nil)     ; list of key-bindings and help strings
 (defun help ()
   (dolist (s *key-bindings*)
     (when (stringp s)
-      (write-string s #|*debug-io*|#))))
+      (safe-wr-st s #|*debug-io*|#))))
 
 (defvar *saved-debug-package* *common-lisp-user-package*)
 (defvar *saved-debug-readtable* (copy-readtable nil))
-(defun debug-reset-io ()
+(defun debug-reset-io (a)
+  (declare (ignore a))
   (rotatef *package* *saved-debug-package*)
   (rotatef *readtable* *saved-debug-readtable*)
   (fresh-line *debug-io*)
@@ -101,138 +110,138 @@
 
 ;; Components of the Break-Loop:
 (defvar *debug-frame*)
-(defvar *debug-mode* 4)
-; lower bound for frame-down/frame-down-1
-(defvar *frame-limit1* nil)
-; upper bound for frame-up and frame-up-1
-(defvar *frame-limit2* nil)
+(defvar *stack-mode* 4)
+; lower bound for frame-down
+(defvar *frame-limit-down* nil)
+; upper bound for frame-up
+(defvar *frame-limit-up* nil)
 
-(defun frame-limit1 (frames-to-skip)
+(defun frame-limit-down (frames-to-skip)
   (let ((frame (the-frame)))
-    (let ((*frame-limit1* nil)
-          (*frame-limit2* nil))
-      (dotimes (i frames-to-skip) (setq frame (frame-up-1 frame 1))))
+    (let ((*frame-limit-down* nil)
+          (*frame-limit-up* nil))
+      (dotimes (i frames-to-skip) (setq frame (frame-up 1 frame 1))))
     frame))
 
-(defun frame-limit2 ()
+(defun frame-limit-up ()
   (let ((frame (the-frame)))
-    (let ((*frame-limit1* nil)
-          (*frame-limit2* nil))
+    (let ((*frame-limit-down* nil)
+          (*frame-limit-up* nil))
       (loop
-       (let ((nextframe (frame-up-1 frame 1)))
+       (let ((nextframe (frame-up 1 frame 1)))
          (when (or (eq nextframe frame) (driver-frame-p nextframe)) (return))
          (setq frame nextframe)))
-      (dotimes (i 2) (setq frame (frame-down-1 frame 1))))
+      (dotimes (i 2) (setq frame (frame-down 1 frame 1))))
     frame))
 
-(defun debug-help () (help) (throw 'debug 'continue))
-(defun debug-unwind () (throw 'debug 'unwind))
-(defun debug-quit () (throw 'debug 'abort-to-top))
-(defun debug-mode-1 () (setq *debug-mode* 1) (throw 'debug 'continue))
-(defun debug-mode-2 () (setq *debug-mode* 2) (throw 'debug 'continue))
-(defun debug-mode-3 () (setq *debug-mode* 3) (throw 'debug 'continue))
-(defun debug-mode-4 () (setq *debug-mode* 4) (throw 'debug 'continue))
-(defun debug-mode-5 () (setq *debug-mode* 5) (throw 'debug 'continue))
+(defun debug-help (a) (declare (ignore a))  (help) (throw 'debug 'continue))
+(defun debug-unwind (a) (declare (ignore a)) (throw 'debug 'unwind))
+(defun debug-quit (a) (declare (ignore a)) (throw 'debug 'abort-to-top))
+(defun stack-mode (argline &key (start 0) (end (length argline)))
+  (multiple-value-bind (mode pos)
+      (read-from-string argline nil *stack-mode* :start start :end end)
+    (check-type mode (integer 1 5))
+    (values mode pos)))
+(defun debug-mode (argline)
+  (setq *stack-mode* (stack-mode argline)) (throw 'debug 'continue))
 
-(defun debug-where ()
+(defun debug-where (a)
+  (declare (ignore a))
   (describe-frame *standard-output* *debug-frame*)
   (throw 'debug 'continue))
 
-(defun debug-up ()
+(defun debug-up (a)
+  (declare (ignore a))
   (describe-frame *standard-output*
-                  (setq *debug-frame* (frame-up-1 *debug-frame* *debug-mode*)))
+                  (setq *debug-frame* (frame-up 1 *debug-frame* *stack-mode*)))
   (throw 'debug 'continue))
 
-(defun debug-top ()
+(defun debug-top (a)
+  (declare (ignore a))
   (describe-frame *standard-output*
-                  (setq *debug-frame* (frame-up *debug-frame* *debug-mode*)))
+                  (setq *debug-frame* (frame-up t *debug-frame* *stack-mode*)))
   (throw 'debug 'continue))
 
-(defun debug-down ()
+(defun debug-down (a)
+  (declare (ignore a))
   (describe-frame *standard-output*
-                  (setq *debug-frame* (frame-down-1 *debug-frame* *debug-mode*)))
+                  (setq *debug-frame* (frame-down 1 *debug-frame*
+                                                  *stack-mode*)))
   (throw 'debug 'continue))
 
-(defun debug-bottom ()
+(defun debug-bottom (a)
+  (declare (ignore a))
   (describe-frame *standard-output*
-                  (setq *debug-frame* (frame-down *debug-frame* *debug-mode*)))
+                  (setq *debug-frame* (frame-down t *debug-frame*
+                                                  *stack-mode*)))
   (throw 'debug 'continue))
 
-(defun get-frame-limit ()
-  (let (number)
-    (loop
-     (fresh-line *debug-io*)
-     (write-string (TEXT "Enter the limit for max. frames to print or ':all' for all: ") *debug-io*)
-     (setq number (read-from-string (read-line *debug-io* nil nil)))
-     (if (or (integerp number) (eq number :all))
-       (return)
-       (progn
-         (fresh-line *debug-io*)
-         (format *debug-io* (TEXT "~A is not a number. Try again.")
-                 number)
-         (elastic-newline *debug-io*))))
-    (unless (eq number :all)
-      number)))
+(defun frame-limit (argline &key (start 0) (end (length argline)))
+  (multiple-value-bind (limit pos)
+      (read-from-string argline nil *debug-print-frame-limit*
+                        :start start :end end)
+    (check-type limit (or null (eql :all) integer))
+    (values (if (eq limit :all) nil limit)
+            pos)))
 
 ;;; sets the limit for frames to print in a backtrace
-(defun debug-set-frame-limit ()
-  (setq *debug-print-frame-limit* (get-frame-limit))
+(defun debug-frame-limit (argline)
+  (setq *debug-print-frame-limit* (frame-limit argline))
   (throw 'debug 'continue))
 
+(defun frame-up-down (limit mode) (frame-down 1 (frame-up 1 limit mode) mode))
+
 (defun print-backtrace (&key ((:out *standard-output*) *standard-output*)
-                        (mode *debug-mode*) (limit *debug-print-frame-limit*))
+                        (mode *stack-mode*) (limit *debug-print-frame-limit*))
   ;; SHOW-STACK prints its output to *STANDARD-OUTPUT*, so we bind that
   (let ((frame-count
          (show-stack
           mode limit
-          (frame-down-1 (frame-up-1 (or *frame-limit1* (frame-limit1 13)) mode)
-                        mode))))
+          (frame-up-down (or *frame-limit-down* (frame-limit-down 13)) mode))))
     (fresh-line *standard-output*)
-    (format *standard-output* (TEXT "Printed ~D frames") frame-count)
+    (format *standard-output* (TEXT "Printed ~D frame~:P") frame-count)
     (elastic-newline *standard-output*)))
 
 ;;; debug-backtrace with-optional 'print-limit'
-(defun debug-backtrace (&optional (mode *debug-mode*)
-                                  (limit *debug-print-frame-limit*)
-                                  (prompt-limit-p nil))
-  (print-backtrace :out *debug-io* :mode mode
-                   :limit (or (and prompt-limit-p (get-frame-limit)) limit))
+(defun debug-backtrace (argline)
+  (multiple-value-bind (mode pos) (stack-mode argline)
+    (print-backtrace :out *debug-io* :mode mode
+                     :limit (frame-limit argline :start pos)))
   (throw 'debug 'continue))
 
-(defun debug-backtrace-1 () (debug-backtrace 1))
-(defun debug-backtrace-2 () (debug-backtrace 2))
-(defun debug-backtrace-3 () (debug-backtrace 3))
-(defun debug-backtrace-4 () (debug-backtrace 4))
-(defun debug-backtrace-5 () (debug-backtrace 5))
-
-(defun debug-trap-on ()
+(defun debug-trap-on (a)
+  (declare (ignore a))
   (trap-eval-frame *debug-frame* t)
   (throw 'debug 'continue))
 
-(defun debug-trap-off ()
+(defun debug-trap-off (a)
+  (declare (ignore a))
   (trap-eval-frame *debug-frame* nil)
   (throw 'debug 'continue))
 
-(defun debug-redo ()
+(defun debug-redo (a)
+  (declare (ignore a))
   (redo-eval-frame *debug-frame*)
   (throw 'debug 'continue))
 
-(defun debug-return ()
-  (return-from-eval-frame *debug-frame* (read-form (TEXT "Values: ")))
+(defun debug-return (argline)
+  (return-from-eval-frame *debug-frame* (read-from-string argline))
   (throw 'debug 'continue))
-(defun debug-continue () (throw 'debug 'quit))
+(defun debug-continue (a) (declare (ignore a)) (throw 'debug 'quit))
 
 ;;; New command to print the error message again
-(defun debug-print-error ()
+(defun debug-print-error (a)
+  (declare (ignore a))
   ;; condition is local to break-loop so have to go back there
   (throw 'debug 'print-error))
-(defun debug-inspect-error ()
+(defun debug-inspect-error (a)
+  (declare (ignore a))
   (throw 'debug 'inspect-error))
 
 ;;; print it
 (defun print-error (condition)
   (fresh-line *debug-io*)
-  (write-string (TEXT "The last error:") *debug-io*)
+  (safe-wr-st (TEXT "The last error:") *debug-io*)
   (pretty-print-condition condition *debug-io* :text-indent 3)
   (elastic-newline *debug-io*))
 
@@ -247,15 +256,15 @@ or a pair (STRING . FUNCTION) so that typing STRING will call FUNCTION.")
             (etypecase binding
               (string binding)
               (cons (cons (car binding)
-                          (lambda ()
-                            (funcall (cdr binding))
+                          (lambda (argline)
+                            (funcall (cdr binding) argline)
                             (throw 'debug 'continue))))))
           (mapcap #'funcall functions)))
 
 ;; extended commands
 (defun commands0 ()
   (list*
-   (TEXT "
+   (TEXT "You are in the top-level Read-Eval-Print loop.
 Help (abbreviated :h) = this list
 Use the usual editing capabilities.
 \(quit) or (exit) leaves CLISP.")
@@ -277,30 +286,21 @@ Unwind         :uw      abort to the next recent input loop
 Reset          :re      toggle *PACKAGE* and *READTABLE* between the
                           local bindings and the sane values
 Quit           :q       quit to the top-level input loop
-Mode-1         :m1      inspect all the stack elements
-Mode-2         :m2      inspect all the frames
-Mode-3         :m3      inspect only lexical frames
-Mode-4         :m4      inspect only EVAL and APPLY frames (default)
-Mode-5         :m5      inspect only APPLY frames
 Where          :w       inspect this frame
 Up             :u       go up one frame, inspect it
 Top            :t       go to top frame, inspect it
 Down           :d       go down one frame, inspect it
 Bottom         :b       go to bottom (most recent) frame, inspect it
-Backtrace-1    :bt1     list all stack elements
-Backtrace-2    :bt2     list all frames
-Backtrace-3    :bt3     list all lexical frames
-Backtrace-4    :bt4     list all EVAL and APPLY frames
-Backtrace-5    :bt5     list all APPLY frames
-Backtrace      :bt      list stack in current mode
-Backtrace-l    :bl      list stack in current mode.
-                          Limit of frames to print will be prompted for.
-Frame-limit    :fl      set the frame-limit. This many frames will
-                          be printed in a backtrace at most.
+Mode mode      :m       set stack mode for Backtrace: 1=all the stack elements
+             2=all the frames                         3=only lexical frames
+             4=only EVAL and APPLY frames (default)   5=only APPLY frames
+Frame-limit n  :fl      set the frame-limit for Backtrace. This many frames
+                          will be printed in a backtrace at most.
+Backtrace [mode [limit]] :bt  inspect the stack
 Break+         :br+     set breakpoint in EVAL frame
 Break-         :br-     disable breakpoint in EVAL frame
 Redo           :rd      re-evaluate form in EVAL frame
-Return         :rt      leave EVAL frame, prescribing the return values")
+Return value   :rt      leave EVAL frame, prescribing the return values")
    (cons "Help"         #'debug-help  )
    (cons ":h"           #'debug-help  )
    (cons "?"            #'debug-help  )
@@ -316,16 +316,6 @@ Return         :rt      leave EVAL frame, prescribing the return values")
    (cons ":re"          #'debug-reset-io)
    (cons "Quit"         #'debug-quit)
    (cons ":q"           #'debug-quit)
-   (cons "Mode-1"       #'debug-mode-1)
-   (cons ":m1"          #'debug-mode-1)
-   (cons "Mode-2"       #'debug-mode-2)
-   (cons ":m2"          #'debug-mode-2)
-   (cons "Mode-3"       #'debug-mode-3)
-   (cons ":m3"          #'debug-mode-3)
-   (cons "Mode-4"       #'debug-mode-4)
-   (cons ":m4"          #'debug-mode-4)
-   (cons "Mode-5"       #'debug-mode-5)
-   (cons ":m5"          #'debug-mode-5)
    (cons "Where"        #'debug-where )
    (cons ":w"           #'debug-where )
    (cons "Up"           #'debug-up    )
@@ -336,22 +326,12 @@ Return         :rt      leave EVAL frame, prescribing the return values")
    (cons ":d"           #'debug-down  )
    (cons "Bottom"       #'debug-bottom)
    (cons ":b"           #'debug-bottom)
-   (cons "Backtrace-1"  #'debug-backtrace-1)
-   (cons ":bt1"         #'debug-backtrace-1)
-   (cons "Backtrace-2"  #'debug-backtrace-2)
-   (cons ":bt2"         #'debug-backtrace-2)
-   (cons "Backtrace-3"  #'debug-backtrace-3)
-   (cons ":bt3"         #'debug-backtrace-3)
-   (cons "Backtrace-4"  #'debug-backtrace-4)
-   (cons ":bt4"         #'debug-backtrace-4)
-   (cons "Backtrace-5"  #'debug-backtrace-5)
-   (cons ":bt5"         #'debug-backtrace-5)
-   (cons "Backtrace"    #'debug-backtrace  )
-   (cons ":bt"          #'debug-backtrace  )
-   (cons "Backtrace-l"  #'(lambda () (debug-backtrace *debug-mode* nil t)))
-   (cons ":bl"          #'(lambda () (debug-backtrace *debug-mode* nil t)))
-   (cons "Frame-limit"  #'debug-set-frame-limit )
-   (cons ":fl"          #'debug-set-frame-limit )))
+   (cons "Mode"         #'debug-mode)
+   (cons ":m"           #'debug-mode)
+   (cons "Frame-limit"  #'debug-frame-limit)
+   (cons ":fl"          #'debug-frame-limit)
+   (cons "Backtrace"    #'debug-backtrace)
+   (cons ":bt"          #'debug-backtrace)))
 
 (defun commands2 ()
   (list
@@ -380,24 +360,38 @@ Over           :o       step over this level: evaluate at once up to the next re
 Continue       :c       switch off single step mode, continue evaluation
 -- Step-until :su, Next-until :nu, Over-until :ou, Continue-until :cu --
            same as above, specify a condition when to stop")
-   (cons "Step"         #'(lambda () (throw 'stepper 'into)))
-   (cons ":s"           #'(lambda () (throw 'stepper 'into)))
-   (cons "Next"         #'(lambda () (throw 'stepper 'over)))
-   (cons ":n"           #'(lambda () (throw 'stepper 'over)))
-   (cons "Over"         #'(lambda () (throw 'stepper 'over-this-level)))
-   (cons ":o"           #'(lambda () (throw 'stepper 'over-this-level)))
-   (cons "Continue"     #'(lambda () (throw 'stepper 'continue)))
-   (cons ":c"           #'(lambda () (throw 'stepper 'continue)))
-   (cons "Step-until"   #'(lambda () (throw 'stepper (values 'into t))))
-   (cons ":su"          #'(lambda () (throw 'stepper (values 'into t))))
-   (cons "Next-until"   #'(lambda () (throw 'stepper (values 'over t))))
-   (cons ":nu"          #'(lambda () (throw 'stepper (values 'over t))))
-   (cons "Over-until"   #'(lambda () (throw 'stepper
-                                       (values 'over-this-level t))))
-   (cons ":ou"          #'(lambda () (throw 'stepper
-                                       (values 'over-this-level t))))
-   (cons "Continue-until" #'(lambda () (throw 'stepper (values 'continue t))))
-   (cons ":cu"          #'(lambda () (throw 'stepper (values 'continue t))))))
+   (cons "Step"         #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper 'into)))
+   (cons ":s"           #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper 'into)))
+   (cons "Next"         #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper 'over)))
+   (cons ":n"           #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper 'over)))
+   (cons "Over"         #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper 'over-this-level)))
+   (cons ":o"           #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper 'over-this-level)))
+   (cons "Continue"     #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper 'continue)))
+   (cons ":c"           #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper 'continue)))
+   (cons "Step-until"   #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper (values 'into t))))
+   (cons ":su"          #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper (values 'into t))))
+   (cons "Next-until"   #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper (values 'over t))))
+   (cons ":nu"          #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper (values 'over t))))
+   (cons "Over-until"   #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper (values 'over-this-level t))))
+   (cons ":ou"          #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper (values 'over-this-level t))))
+   (cons "Continue-until" #'(lambda (a) (declare (ignore a))
+                                    (throw 'stepper (values 'continue t))))
+   (cons ":cu"          #'(lambda (a) (declare (ignore a))
+                                  (throw 'stepper (values 'continue t))))))
 
 (defun commands (may-continue commandsr)
   (nconc (commands1)
@@ -408,24 +402,27 @@ Continue       :c       switch off single step mode, continue evaluation
          commandsr
          (wrap-user-commands *user-commands*)))
 
+;; establish the DEBUG catcher and the ABORT restart
+(defmacro with-abort-restart ((&key report) &body body)
+  `(catch 'debug
+     (with-restarts ((ABORT () :report (lambda (s) (write-string ,report s))
+                       (throw 'debug 'continue)))
+       ,@body)))
+
 ;; Main-Loop with additional help-command
 (defun main-loop ()
   (setq *break-count* 0)
   (driver                 ; build driver-frame; do #'lambda "infinitely"
    #'(lambda ()
-       (catch 'debug            ; catch the (throw 'debug ...)
+       (with-abort-restart (:report (TEXT "Abort main loop"))
          ;; ANSI CL wants an ABORT restart to be available.
-         (with-restarts ((ABORT () (throw 'debug 'continue)))
-           (when (read-eval-print   ; read-eval-print INPUT-line
-                   (string-concat (prompt-start) (prompt-body) (prompt-finish))
-                   (commands0))
-             ;; T -> #<EOF>
-             ;; NIL -> form is already evaluated
-             ;;        result has been printed
-             (if (interactive-stream-p *standard-input*)
-               (exit)           ; user typed EOF
-               (progn (setq *command-index* 0) ; reset *command-index*
-                      (return-from main-loop))))))))) ; and proceed
+         (when (read-eval-print   ; read-eval-print INPUT-line
+                (string-concat (prompt-start) (prompt-body) (prompt-finish))
+                (commands0))
+           ;; T -> #<EOF>
+           ;; NIL -> form is already evaluated
+           ;;        result has been printed
+           (exit))))))
 
 (setq *driver* #'main-loop)
 
@@ -463,7 +460,7 @@ Continue       :c       switch off single step mode, continue evaluation
     (if may-continue
       (progn
         (write-string "** - " *error-output*)
-        (write-string (TEXT "Continuable Error") *error-output*)
+        (safe-wr-st (TEXT "Continuable Error") *error-output*)
         (terpri *error-output*))
       (write-string "*** - " *error-output*))
 
@@ -472,7 +469,7 @@ Continue       :c       switch off single step mode, continue evaluation
       (if (> *recursive-error-count* 3)
         (progn
           (setq *recursive-error-count* 0)
-          (write-string (TEXT "Unprintable error message.")
+          (write-string (TEXT "Unprintable error message")
                         *error-output*))
         (pretty-print-condition condition *error-output*
                                 :text-indent (if may-continue 5 6))))
@@ -484,14 +481,14 @@ Continue       :c       switch off single step mode, continue evaluation
       (if continuable
         (when interactive-p
           (fresh-line *debug-io*)
-          (write-string (TEXT "You can continue (by typing 'continue').")
-                        *debug-io*)
+          (safe-wr-st (TEXT "You can continue (by typing 'continue').")
+                      *debug-io*)
           (elastic-newline *debug-io*))
         (progn
           (fresh-line *debug-io*)
           (when interactive-p
-            (write-string (TEXT "If you continue (by typing 'continue'): ")
-                          *debug-io*))
+            (safe-wr-st (TEXT "If you continue (by typing 'continue'): ")
+                        *debug-io*))
           (princ may-continue *debug-io*)
           (elastic-newline *debug-io*)))))
 
@@ -503,7 +500,7 @@ Continue       :c       switch off single step mode, continue evaluation
       (when restarts
         (when interactive-p
           (fresh-line *debug-io*)
-          (write-string restarts-help *debug-io*)
+          (safe-wr-st restarts-help *debug-io*)
           (elastic-newline *debug-io*))
         (let ((counter 0))
           (dolist (restart restarts)
@@ -513,11 +510,12 @@ Continue       :c       switch off single step mode, continue evaluation
                    (helpstring (format nil "~A~15T~A~24T~A" name command
                                        (princ-to-string restart)))
                    (restart restart)  ; for FUNC
-                   (func #'(lambda () (invoke-restart-interactively restart))))
+                   (func #'(lambda (a) (declare (ignore a))
+                                   (invoke-restart-interactively restart))))
               ;; display the restarts:
               (when interactive-p
                 (fresh-line *debug-io*)
-                (write-string helpstring *debug-io*)
+                (safe-wr-st helpstring *debug-io*)
                 (elastic-newline *debug-io*))
               (push (string-concat (string #\Newline) helpstring) commandsr)
               ;; put it into the commandsr list.
@@ -537,29 +535,27 @@ Continue       :c       switch off single step mode, continue evaluation
            (*standard-output* stream)
            (prompt (string-concat (prompt-start) (prompt-break)
                                   (prompt-body) (prompt-finish)))
-           (*frame-limit1* (frame-limit1 13))
-           (*frame-limit2* (frame-limit2))
-           (*debug-mode* *debug-mode*)
-           (*debug-frame*
-            (frame-down-1 (frame-up-1 *frame-limit1* *debug-mode*)
-                          *debug-mode*))
+           (*frame-limit-down* (frame-limit-down 13))
+           (*frame-limit-up* (frame-limit-up))
+           (*stack-mode* *stack-mode*)
+           (*debug-frame* (frame-up-down *frame-limit-down* *stack-mode*))
+           (*saved-debug-package* *saved-debug-package*)
+           (*saved-debug-readtable* *saved-debug-readtable*)
            (commands-list (commands may-continue commandsr)))
       (driver
        ;; build driver frame and repeat #'lambda (infinitely; ...)
        #'(lambda ()
-           (case (catch 'debug    ; catch (throw 'debug ...) and analyse
+           (case (with-abort-restart (:report (TEXT "Abort debug loop"))
                    ;; ANSI CL wants an ABORT restart to be available.
-                   (with-restarts ((ABORT () (throw 'debug 'continue)))
-                     ;; build environment *debug-frame*
-                     ;; which is valid/equal for/to *debug-frame*
-                     (same-env-as *debug-frame*
-                       #'(lambda ()
-                           (if (read-eval-print ; read-eval-print INPUT-line
-                                 prompt commands-list)
-                             ;; T -> #<EOF>
-                             ;; NIL -> form is already evaluated;
-                             ;;        result has been printed
-                             (throw 'debug (if may-continue 'quit 'unwind)))))))
+                   ;; build environment *debug-frame*
+                   ;; which is valid/equal for/to *debug-frame*
+                   (same-env-as *debug-frame*
+                     #'(lambda ()
+                         (if (read-eval-print prompt commands-list)
+                           ;; T -> #<EOF>
+                           ;; NIL -> form is already evaluated;
+                           ;;        result has been printed
+                           (throw 'debug (if may-continue 'quit 'unwind))))))
              ((print-error) (print-error condition))
              ((inspect-error) (inspect condition))
              ((unwind) (go unwind))
@@ -596,15 +592,15 @@ Continue       :c       switch off single step mode, continue evaluation
 (defun step-values (values)
   (let ((*standard-output* *debug-io*))
     (fresh-line #|*debug-io*|#)
-    (write-string (TEXT "step ") #|*debug-io*|#)
+    (safe-wr-st (TEXT "step ") #|*debug-io*|#)
     (write *step-level* #|:stream *debug-io*|#)
     (write-string " ==> " #|*debug-io*|#)
     (case (length values)
-      (0 (write-string (TEXT "no values") #|*debug-io*|#))
-      (1 (write-string (TEXT "value: ") #|*debug-io*|#)
+      (0 (safe-wr-st (TEXT "no values") #|*debug-io*|#))
+      (1 (safe-wr-st (TEXT "value: ") #|*debug-io*|#)
          (write (car values) #|:stream *debug-io*|#))
       (t (write (length values) #|:stream *debug-io*|#)
-         (write-string (TEXT " values: ") #|*debug-io*|#)
+         (safe-wr-st (TEXT " values: ") #|*debug-io*|#)
          (do ((L values))
              ((endp L))
            (write (pop L) #|:stream *debug-io*|#)
@@ -631,15 +627,13 @@ Continue       :c       switch off single step mode, continue evaluation
              (*standard-output* stream)
              (prompt (string-concat (prompt-start) (prompt-step)
                                     (prompt-body) (prompt-finish)))
-             (*frame-limit1* (frame-limit1 11))
-             (*frame-limit2* (frame-limit2))
-             (*debug-mode* *debug-mode*)
-             (*debug-frame*
-              (frame-down-1 (frame-up-1 *frame-limit1* *debug-mode*)
-                            *debug-mode*))
+             (*frame-limit-down* (frame-limit-down 11))
+             (*frame-limit-up* (frame-limit-up))
+             (*stack-mode* *stack-mode*)
+             (*debug-frame* (frame-up-down *frame-limit-down* *stack-mode*))
              (commands-list (commands nil (commands4))))
         (fresh-line #|*debug-io*|#)
-        (write-string (TEXT "step ") #|*debug-io*|#)
+        (safe-wr-st (TEXT "step ") #|*debug-io*|#)
         (write *step-level* #|:stream *debug-io*|#)
         (write-string " --> " #|*debug-io*|#)
         (write form #|:stream *debug-io*|# :length 4 :level 3)
@@ -651,22 +645,19 @@ Continue       :c       switch off single step mode, continue evaluation
                   ;;  build driver frame and repeat #'lambda (infinitely ...)
                   #'(lambda ()
                       ;; catch the (throw 'debug ...) and analyse
-                      (case
-                          (catch 'debug
-                            ;; ANSI CL wants an ABORT restart to be available.
-                            (with-restarts ((ABORT () (throw 'debug 'continue)))
+                      (case (with-abort-restart (:report (TEXT "Abort stepper"))
+                              ;; ANSI CL wants an ABORT restart to be available.
                               (same-env-as *debug-frame*
                                 ;; build environment *debug-frame* that
                                 ;; is valid/equal for/to *debug-frame*
                                 #'(lambda ()
-                                    (if (read-eval-print ; get/read INPUT-line
-                                          prompt commands-list)
+                                    (if (read-eval-print prompt commands-list)
                                       ;; T -> #<EOF>
                                       (go continue)
                                       ;; NIL -> form is already evaluated;
                                       ;;        result has been printed
                                       #|(throw 'debug 'continue)|#
-                                      )))))
+                                      ))))
                         ((unwind) (go unwind))
                         ((abort-to-top) (go abort-to-top))
                         (t ))))) ; other cases, especially continue
