@@ -26,15 +26,18 @@
 
 ;; Saves the current memory contents.
 ;; This function works only when compiled!
+(defvar *saveinitmem-verbose* t
+  "The default value for the :VERBOSE argument of SAVEINITMEM.")
 (defun saveinitmem (&optional (filename "lispinit.mem")
-                    &key ((:quiet *quiet*) nil) init-function verbose
+                    &key ((:quiet *quiet*) nil) init-function
+                    ((:verbose *saveinitmem-verbose*) *saveinitmem-verbose*)
                     ((:norc *norc*) nil)
                     ((:documentation *image-doc*)
                      (documentation init-function 'function))
                     ((:script *script*) (null init-function))
                     keep-global-handlers (start-package *package*)
                     (locked-packages *system-package-list*) executable)
-  (let* ((old-driver *driver*) old-global-handlers
+  (let* ((old-driver *driver*) old-global-handlers file-size
          (*package* (sys::%find-package start-package))
          (fn (if (not executable)
                (merge-pathnames filename #.(make-pathname :type "mem"))
@@ -44,8 +47,7 @@
                #-(or win32 cygwin) filename))
          (*driver*
            #'(lambda ()
-               ;(declare (special *command-index* *home-package*
-               ;                  *active-restarts* *condition-restarts*))
+               ;(declare (special *command-index* *home-package*))
                ;; Reset a few special variables. This must happen in the
                ;; fresh session, not in the old session, because that would
                ;; temporarily disable error handling in the old session.
@@ -62,8 +64,6 @@
                      / nil
                      // nil
                      /// nil
-                     *active-restarts* nil
-                     *condition-restarts* nil
                      *command-index* 0
                      *home-package* nil)
                (setq *driver* old-driver)
@@ -73,11 +73,16 @@
       (setq old-global-handlers ; disable and save all global handlers
             (ext::set-global-handler nil nil)))
     (setf (package-lock locked-packages) t)
-    (savemem fn executable)
+    (let ((*active-restarts* nil)
+          (*condition-restarts* nil))
+      ;; we used to set *ACTIVE-RESTARTS* & *CONDITION-RESTARTS* above in the
+      ;; *DRIVER* binding, but that caused mutiple ABORT restarts, bug
+      ;; http://sourceforge.net/tracker/index.php?func=detail&aid=1877497&group_id=1355&atid=101355
+      (setq file-size (savemem fn executable)))
     (when old-global-handlers   ; re-install all global handlers
       (ext::set-global-handler old-global-handlers nil))
-    (when verbose
-      (fresh-line)
-      (format t (TEXT "Wrote the memory image into ~A") fn)
-      (terpri)))
+    (when *saveinitmem-verbose*
+      (let ((*load-level* 1))   ; proper indentation
+        (loading-message (TEXT "Wrote the memory image into ~A (~:D byte~:P)")
+                         fn file-size))))
   (room nil))
