@@ -85,7 +85,7 @@ The input file is normal C code, modified like this:
  enum means no #ifdef
  prefix, suffix default to ""
  reverse defaults to "" and means ERROR
- bitmasks means additional *_to_list and *_from_list functions are defined
+ bitmasks means additional *_to_list and *_of_list functions are defined
  delim defaults to "_" and separates prefix and suffix
 
 Restrictions and caveats:
@@ -243,13 +243,38 @@ The vector is freshly constructed, but the strings are shared"
               (let ((start (next-non-blank argument (1+ =))))
                 (if start (subseq argument start) T)))
         argument)))
-(defun split-command (line &key (start 0) (end (length line)))
+(defun strip-comments (line start end)
+  "Return the same LINE but without the embedded comments."
+  ;; abc/*c1*/def/*c2 --> abcdef
+  (do ((comment-start (search #1="/*" line :start2 start :end2 end)
+                      (and comment-end
+                           (search #1# line :start2 comment-end :end2 end)))
+       parts (comment-end start))
+      ((null comment-start)
+       (if parts
+           (let ((ret (apply #'ext:string-concat
+                             (nreverse (if comment-end
+                                           (cons (subseq line comment-end
+                                                         ;; include last #\)
+                                                         (and end (1+ end)))
+                                                 parts)
+                                           parts)))))
+             (values ret 0 (position #\) ret :from-end t)))
+           (values line start end)))
+    (push (subseq line comment-end comment-start) parts)
+    (when (setq comment-end (search #2="*/" line :start2 comment-start
+                                    :end2 end))
+      (incf comment-end #.(length #2#)))))
+(defun split-command (line &key (start 0) (end (length line)) &aux end1)
   "parse command line into command name and arguments:
 FOO(bar,baz,zot) ==> FOO; (bar baz zot); end-position"
+  ;; end: where the command in the original line in *lines* ends
+  ;; end1: where the command in the comment-stripped line ends
   (setq start (next-non-blank line start))
   (unless start (return-from split-command nil)) ; blank line
   (setq end (position #\) line :start start :end end :from-end t))
-  (let* ((last (or (prev-non-blank line end)         ;nothing before #\)
+  (setf (values line start end1) (strip-comments line start end))
+  (let* ((last (or (prev-non-blank line end1) ; nothing before #\)
                    (return-from split-command nil))) ; => no command
          (paren (position #\( line :start start :end last)) args
          (name (subseq line start
@@ -259,7 +284,7 @@ FOO(bar,baz,zot) ==> FOO; (bar baz zot); end-position"
       (return-from split-command nil))
     ;; valid command in LINE
     (unless end (error "~S:~D: no closing paren in ~S[~:D;~:D]"
-                       *input-file* *lineno* line start end))
+                       *input-file* *lineno* line start end1))
     (unless paren (error "~S:~D: no opening paren in ~S[~:D;~:D]"
                          *input-file* *lineno* line start last))
     (setq start (next-non-blank line (1+ paren)))
@@ -334,8 +359,9 @@ See bug #[ 1491252 ]: i18n does not build on cf:alpha")
                              (write-char #\_ s)
                              (write-string-c-style (cdr el) s)))))))))
     (when (< *tag-length-limit* (length base))
-      (warn "truncated a very long tag (from ~:d to ~:d) for ~s"
-            (length base) *tag-length-limit* init)
+      (let ((*print-lines* 5))
+        (warn "truncated a very long tag (from ~:d to ~:d) for ~s"
+              (length base) *tag-length-limit* init))
       (setq base (subseq base 0 *tag-length-limit*)))
     (when (funcall already-present-p base)
       (loop :for ii :upfrom 0 :for new = (format nil "~a_~d" base ii)
@@ -1099,7 +1125,7 @@ commas and parentheses."
             (when bitmasks
               (formatln out "#define ~A_to_list(a) map_c_to_list(a,&~A_map)"
                         c-name c-name)
-              (formatln out "#define ~A_from_list(a) map_list_to_c(a,&~A_map)"
+              (formatln out "#define ~A_of_list(a) map_list_to_c(a,&~A_map)"
                         c-name c-name))))
     (newline out)))
 
